// Class: PattViewer
//
// Purpose: Creates PattFrames and allows simultaneous manipulation
//
// History: v1.0 7/03 WFS
//          v1.1 6/06 WFS
//
// COPYRIGHT, 2003, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 

import java.util.LinkedList;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.JPanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import viewer.*;

public class PattViewer extends JFrame{
    public static int version = 200301;
    public static String copyright = "Copyright 2003, Fox Chase Cancer Center";
    
    JButton left=new JButton("<<");
    JButton right=new JButton(">>");
    JButton quit=new JButton("Quit");
    JButton newPatt=new JButton("New Frame");
    JPanel panel=new JPanel();

    LinkedList<PattFrame> pattList=new LinkedList<PattFrame>();//contains all PattFrames created

    public PattViewer(){
	try{
	    pattInit();
	}catch(Exception exception){
	    exception.printStackTrace();
	}
    }

    /*
    * initializes the gui and sets the action listeners
    */
    private void pattInit() throws Exception{
	this.setTitle("PattViewer");
	this.setSize(750,85);
	this.getContentPane().setLayout(new BorderLayout());
	this.setDefaultCloseOperation(3);

	quit.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent e){
		System.exit(0);
	    }
	});
	quit.setPreferredSize(new Dimension(75,25));

	newPatt.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent e){
		PattFrame temp=new PattFrame();
		temp.setVisible(true);
		pattList.add(temp);
	    }
	});
	newPatt.setPreferredSize(new Dimension(120,25));

	//Listeners which invoke either the left or right methods on all PattFrames
	left.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent e){
		Iterator itr=pattList.iterator();
		while(itr.hasNext()){
		    PattFrame temp=(PattFrame)itr.next();
		    if(temp.isDisplayable()){
			if(temp.left.isVisible()&&temp.left.isEnabled()){
			    temp.rl_actionPerformed(false);
			}
		    }else{
			itr.remove();
		    }
		}
	    }
	});
	left.setPreferredSize(new Dimension(60,50));

	right.addActionListener(new ActionListener(){
	    public void actionPerformed(ActionEvent e){
		Iterator itr=pattList.iterator();
		while(itr.hasNext()){
		    PattFrame temp=(PattFrame)itr.next();
		    if(temp.isDisplayable()){
			if(temp.right.isVisible()&&temp.right.isEnabled()){
			    temp.rl_actionPerformed(true);
			}
		    }else{
			itr.remove();
		    }
		}
	    }
	});
	right.setPreferredSize(new Dimension(60,50));

	panel.add(quit,null);
	panel.add(newPatt,null);
	panel.add(left,null);
	panel.add(right,null);
	this.getContentPane().add(panel,BorderLayout.SOUTH);
    }

    /*
    * creates the PattViewer
    */
    public static void main(String arg[]){
	PattViewer newPatt=new PattViewer();
	newPatt.setVisible(true);
    }
}
