// Class: LSNMF
//
// Purpose: java wrapper around libLSNMF.jnilib
//
// History v1.0 7/06 WFS
//
// COPYRIGHT, 2006, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have recieved a copy of the GNU General Public License
// along with this program;  if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA   02111-1307, USA

package lib;

import java.io.IOException;

import pattviewer.PattInput;
import pattviewer.GenpatExp;

public class LSNMF extends Library{
    public static final String name = "LSNMF";
    public static final String lib = "LSNMF";


    private native int lsnmf(PattInput pattIn, GenpatExp pattOut);

    public LSNMF()throws SecurityException, IOException, UnsatisfiedLinkError{
	super();
    }

    public String[] getFieldNames(){
	return new String[]{"tag","dir","ENSEMBLE","Niter","Nshow","Iseed","AlphaA","AlphaF","lineMeasure",
			    "Na","Nf","ptwidth","data","acc","xLabels","geneNames","uncertFlag",
			    "StartRank","EndRank","Nchains"};
    }

    public String getLib(){
	return lib;
    }

    public String getName(){
	return name;
    }

    public static void validateInput(PattInput input)throws Exception{
	Library.validateInput(input);

	if(input.getStartRank()<0)
	    throw new Exception("StartRank must be >= 0");
    }

    public GenpatExp run(PattInput pattIn){
	pattIn.setENSEMBLE(1);
	GenpatExp pattOut = new GenpatExp();
	try{
	    validateInput(pattIn);
	    lsnmf(pattIn, pattOut);
	}catch(Throwable e){
	    guiPrint(e.toString());
	}
	return pattOut;
    }
}
