#include "Common.h"

////////////////////////////////////////////////////
int Common::Zscoring(double *orig, double *zscore, int N)
////////////////////////////////////////////////////

{
	double sum1 = 0.0;
	double sum2 = 0.0;

	if(N<=1) return(-1);

	for (int i=0; i<N; i++)
	{
		sum1 += orig[i];
		sum2 += orig[i] * orig[i];
	}

	double sd = (sum2 - (sum1*sum1)/N) / (N-1);
	if(fabs(sd) < 1e-15 || sd < 0.0) {
		for (int i=0; i<N; i++) zscore[i] = 0.0;
		return(0);
	}

	sd = sqrt(sd);
	double average = sum1 / N;

	for (int i=0; i<N; i++) zscore[i] = (orig[i] - average) / sd;

	return(1);
}

////////////////////////////////////////////////////
int Common::Zscoring(double *orig, int offset, double *zscore, int N)
////////////////////////////////////////////////////

{
	double sum1 = 0.0;
	double sum2 = 0.0;

	if(N<=1) return(-1);

	offset *= N;

	for (int i=offset; i<N+offset; i++)
	{
		sum1 += orig[i];
		sum2 += orig[i] * orig[i];
	}

	double sd = (sum2 - (sum1*sum1)/N) / (N-1);
	if(fabs(sd) < 1e-15 || sd < 0.0) {
		for (int i=0; i<N; i++) zscore[i] = 0.0;
		return(0);
	}

	sd = sqrt(sd);
	double average = sum1 / N;

	for (int i=0; i<N; i++) zscore[i] = (orig[offset + i] - average) / sd;

	return(1);
}

///////////////////////////////////////////////////////////////////////////////////
// ran2 provided a better portable random number generators (0,1), which
// library function rand() might not.
// For using rand(), only the first 1/10 of RAND_MAX numbers are
// safely random. For most of the current machines, RAND_MAX = 2147483647,
// which means about 150 millions numbers generated from rand() would be good. 
///////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////
double Common::ran2(long *idum)
/////////////////////////////////
{
	int   j;
	long  k;
	static long idum2 = 123456789;
	static long iy = 0;
	static int    IM1 = 2147483563;
	static int    IMM1 = IM1 - 1;
	static int    IM2 = 2147483399;
	static double AM  = 1.0 / (double)IM1;
	static int    IA1 = 40014;
	static int    IA2 = 40692;
	static int    IQ1 = 53668;
	static int    IQ2 = 52774;
	static int    IR1 = 12211;
	static int    IR2 = 3791;
	const  int    NTAB = 32;
	static int    NDIV = 1 + IMM1 / NTAB;
	static double EPS = 1.2e-7;
	static double RNMX = 1.0 - EPS;
	static long iv[NTAB];
	double temp;

	if (*idum <= 0) 
	{
		if (-(*idum) < 1) *idum = 1; else *idum = -1 * (*idum);
		idum2 = (*idum);

		for (j=NTAB+7; j>=0; j--)
		{
			k = (*idum) / IQ1;
			*idum = IA1 * (*idum - k * IQ1) - k * IR1;

			if(*idum < 0) *idum += IM1;
			if(j < NTAB) iv[j] = *idum;
		}

		iy = iv[0];
	}

	k = (*idum) / IQ1;
	*idum = IA1 * (*idum - k * IQ1) - k * IR1;

	if(*idum < 0) *idum += IM1;

	k = idum2 / IQ2;
	idum2 = IA2 * (idum2 - k * IQ2) - k * IR2;

	if(idum2 < 0) idum2 += IM2;

	j = iy / NDIV;
	iy = iv[j] - idum2;
	iv[j] = *idum;

	if(iy < 1) iy += IMM1;
	if( (temp = AM * iy) > RNMX) return RNMX; else return temp;
}
