// Class: GenpatExp
//
// Purpose: serialized class containing output data from algorithms
//
// History: v1.0 6/01 MFO
//          v1.1 7/03 WFS
//          v2.0 7/06 WFS
//
// COPYRIGHT, 2006, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package pattviewer;

import java.io.Serializable;

public class GenpatExp implements Serializable
{
    //Arrays to be used in this class
    double footprint[];       //kernel convolution function
    double distribution[][];  //A matrix output
    double distSD[][];        //std dev for distribution
    double pattSD[][];        //std dev for pattern
    double pattern[][];       //F matrix output
    double data[][];          //input data
    double mock[][];          //data reconstructed from model
    double noise[][];         //noise levels of input data
    
    //Integers
    int Nsol;    //number of solutions posited in BD
    int Na;      //number of rows in data matrix
    int Nf;      //number of columns in data matrix
    int Nfoot;   //size of kernel
    int ENSEMBLE;
    int Niter;
    int Nshow;
    int Iseed;
    int SeedP;
    
    //doubles
    double chisq;
    double AtomsA;
    double AtomsF;
    double Evidence;
    double ptwidth;
    double lineMeasure;
    
    //Strings
    String tag;                //tag identifying experiment
    String date;               //date run completed
    String[] xLabels;
    String[] geneNames;
    
    //Constructor for class
    public GenpatExp()
    {
	super();
	clear();
    }

    public void clear(){
	footprint = new double[0];
	distribution = new double[0][0];
	distSD = new double[0][0];
	pattSD = new double[0][0];
	pattern = new double[0][0];
	data = new double[0][0];
	mock = new double[0][0];
	noise = new double[0][0];
	Nsol = 0;
	Na = 0;
	Nf = 0;
	Nfoot = 0;
	ENSEMBLE = 0;
	Niter = 0;
	Nshow = 0;
	Iseed = 0;
	SeedP = 0;
	chisq = 0;
	AtomsA = 0;
	AtomsF = 0;
	Evidence = 0;
	ptwidth = 0;
	lineMeasure = 0;
	tag = "";
	date = "";
	xLabels = new String[0];
	geneNames= new String[0];
    }

    /*
     * stores a copy of the data in copy
     */
    public void set(GenpatExp b){
	footprint = b.getFootprint().clone();
	distribution = b.getDistribution().clone();
	distSD = b.getDistSD().clone();
	pattern = b.getPattern().clone();
	pattSD = b.getPattSD().clone();
	data = b.getData().clone();
	mock = b.getMock().clone();
	noise = b.getNoise().clone();
	Nsol = b.getNsol();
	Na = b.getNa();
	Nf = b.getNf();
	Nfoot = b.getNfoot();
	ENSEMBLE = b.getENSEMBLE();
	Niter = b.getNiter();
	Nshow = b.getNshow();
	Iseed = b.getIseed();
	SeedP = b.getSeedP();
	chisq = b.getChisq();
	AtomsA = b.getAtomsA();
	AtomsF = b.getAtomsF();
	Evidence = b.getEvidence();
	ptwidth = b.getPtwidth();
	lineMeasure = b.getLineMeasure();
	tag = b.getTag();
	date = b.getDate();
	xLabels = b.getXLabels().clone();
	geneNames = b.getGeneNames().clone();
    }
    
    //Get and Set Arrays
    //footprint
    public void setFootprint (double[] input) {
	footprint = input;
	return;
    }
    
    public double[] getFootprint () {
	return footprint;
    }
    //=====================================================
    //date
    public void setDate (String input) {
	date = input;
	return;
    }

    public String getDate () {
	return date;
    }
    //=====================================================
    //distribution
    public void setDistribution (double[][] input) {
	distribution = input;
	return;
    }
    
    public double[][] getDistribution () {
	return distribution;
    }
    //=====================================================
    //pattern
    public void setPattern (double[][] input) {
	pattern = input;
	return;
    }
    
    public double[][] getPattern () {
	return pattern;
    }
    //=====================================================
    //data
    public void setData (double[][] input) {
	data = input;
	return;
    }
    
    public double[][] getData () {
	return data;
    }
    //=====================================================
    //mock
    public void setMock (double[][] input) {
	mock = input;
	return;
    }
    
    public double[][] getMock () {
	return mock;
    }
    //=====================================================
    //noise
    public void setNoise (double[][] input) {
	noise = input;
	return;
    }

    public double[][] getNoise () {
	return noise;
    }
    //======================================================
    //distSD
    public void setDistSD (double[][] input){
	distSD = input;
	return;
    }
    
    public double[][] getDistSD () {
	return distSD;
    }
    //======================================================
    //pattSD
    public void setPattSD (double[][] input) {
	pattSD = input;
	return;
    }
    
    public double[][] getPattSD () {
	return pattSD;
    }
    //======================================================
    //Nsol
    public void setNsol (int input) {
	Nsol = input;
	return;
    }

    public int getNsol () {
	return Nsol;
    }
    //======================================================
    //Na
    public void setNa (int input) {
	Na = input;
	return;
    }

    public int getNa () {
	return Na;
    }
    //======================================================
    //Nf
    public void setNf (int input) {
	Nf = input;
	return;
    }

    public int getNf () {
	return Nf;
    }
    //======================================================
    //Nfoot
    public void setNfoot (int input) {
	Nfoot = input;
	return;
    }
    
    public int getNfoot () {
	return Nfoot;
    }
    //======================================================
    //ENSEMBLE
    public void setENSEMBLE (int input) {
	ENSEMBLE = input;
	return;
    }
    
    public int getENSEMBLE () {
	return ENSEMBLE;
    }
    //=======================================================
    //Niter
    public void setNiter (int input) {
	Niter = input;
	return;
    }
    
    public int getNiter () {
	return Niter;
    }
    //=======================================================
    //Nshow
    public void setNshow (int input) {
	Nshow = input;
	return;
    }

    public int getNshow () {
	return Nshow;
    }
    //=======================================================
    //Iseed
    public void setIseed (int input) {
	Iseed = input;
	return;
    }
    
    public int getIseed () {
	return Iseed;
    }
    //=======================================================
    //SeedP
    public void setSeedP (int input) {
	SeedP = input;
	return;
    }
    
    public int getSeedP () {
	return SeedP;
    }
    //=======================================================
    //chisq
    public void setChisq (double input) {
	chisq = input;
	return;
    }
    
    public double getChisq () {
	return chisq;
    }
    //=======================================================
    //AtomsA
    public void setAtomsA (double input) {
	AtomsA = input;
	return;
    }
    
    public double getAtomsA () {
	return AtomsA;
    }
    //=======================================================
    //AtomsF
    public void setAtomsF (double input) {
	AtomsF = input;
	return;
    }
    
    public double getAtomsF () {
	return AtomsF;
    }
    //=======================================================
    //Evidence
    public void setEvidence (double input) {
	Evidence = input;
	return;
    }
    
    public double getEvidence () {
	return Evidence;
    }
    //=======================================================
    //ptwidth
    public void setPtwidth (double input) {
	ptwidth = input;
	return;
    }
    
    public double getPtwidth () {
	return ptwidth;
    }
    //=======================================================
    //lineMeasure
    public void setLineMeasure (double input) {
	lineMeasure = input;
	return;
    }
    
    public double getLineMeasure () {
	return lineMeasure;
    }
    //=======================================================
    //tag
    public void setTag (String input) {
	tag = input;
	return;
    }
    
    public String getTag () {
	return tag;
    }
    //=====================================================
    //X labels
    public void setXLabels (String[] input) {
	xLabels = input;
	return;
    }
    
    public String[] getXLabels () {
	return xLabels;
    }
    //=====================================================
    //gene names
    public void setGeneNames(String[] input){
	geneNames = input;
    }

    public String[] getGeneNames(){
	return geneNames;
    }    
}
