// Class: PattInput
//
// Purpose: serializable object containing all of the data bd takes as input
//
// History: v0.1 5/06 WFS
//
// COPYRIGHT, 2006, Fox Chase Cancer Center
//
// This program is free software; you can redistribute is and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package pattviewer;

import java.io.Serializable;

public class PattInput implements Serializable{
    public static int version = 200601;
    public static String copyright = "Copyright 2006, Fox Chase Cancer Center";

    //data fields from .ctrl file
    public int ENSEMBLE, Niter, Nshow, Iseed;
    public double AlphaA, AlphaF, lineMeasure;
    public String tag;
    public String dir;

    //data fields from .data, .tab, or .txt file
    public int Nsol, Na, Nf;
    public double ptwidth;
    public double[] data, acc;
    public double additive;         // additive error if acc is not set
    public double multiplicative;   // multiplicative error if acc is not set
    public String[] xLabels, geneNames;

    //data fields from .link file
    public double SPLIT_portion;
    public int NfeetA;
    public int[] NfootA;
    public int[][] NlistA;
    public int Wtype;

    //data fields from .cls file
    public int Nfeet;
    public int[] feetsizes;

    //data fields from .ctr file
    public int uncertFlag, StartRank, EndRank, Nchains;

    //set all of the fields to 0 or empty arrays by default
    public PattInput(){
	clear();
    }

    public void clear(){
	ENSEMBLE=1;
	Niter=10000;
	Nshow=0;
	Iseed=0;
	AlphaA=0d;
	AlphaF=0d;
	lineMeasure=0d;
	tag="";
	dir="";
	Nsol=0;
	Na=0;
	Nf=0;
	ptwidth=0;
	data=new double[0];
	acc=new double[0];
	additive = 0.01;
	multiplicative = 0.1;
	xLabels=new String[0];
	geneNames=new String[0];
	SPLIT_portion=0;
	NfeetA=0;
	NfootA=new int[0];
	NlistA=new int[0][0];
	Wtype=0;
	Nfeet=0;
	feetsizes=new int[0];
	uncertFlag=0;
	StartRank=4;
	EndRank=6;
	Nchains=10;
    }

    public void set(PattInput b){
	ENSEMBLE=b.getENSEMBLE();
	Niter = b.getNiter();
	Nshow = b.getNshow();
	Iseed = b.getIseed();
	AlphaA = b.getAlphaA();
	AlphaF = b.getAlphaF();
	lineMeasure = b.getLineMeasure();
	tag = b.getTag();
	dir = b.getDir();
	Nsol = b.getNsol();
	Na = b.getNa();
	Nf = b.getNf();
	ptwidth = b.getPtwidth();
	data = b.getData().clone();
	acc = b.getAcc().clone();
	additive = b.getAdditive();
	multiplicative = b.getMultiplicative();
	xLabels = b.getXLabels().clone();
	geneNames = b.getGeneNames().clone();
	SPLIT_portion = b.getSPLIT_portion();
	NfeetA = b.getNfeetA();
	NfootA = b.getNfootA().clone();
	NlistA = b.getNlistA().clone();
	Wtype = b.getWtype();
	Nfeet = b.getNfeet();
	feetsizes = b.getFeetsizes().clone();
	uncertFlag = b.getUncertFlag();
	StartRank = b.getStartRank();
	EndRank = b.getEndRank();
	Nchains = b.getNchains();
    }

    public boolean constrain(String field){
	if((field.equals("Na")||
	   field.equals("Nf")||
	   field.equals("Nsol"))&&
	   !(AlphaA==Na*Nsol&&
	   AlphaF==Nf*Nsol)){
	    AlphaA = Na*Nsol;
	    AlphaF = Nf*Nsol;
	    return true;
	}
	return false;
    }

    //get and set data
    //ENSEMBLE
    public void setENSEMBLE(int E){
	ENSEMBLE=E;
    }

    public int getENSEMBLE(){
	return ENSEMBLE;
    }
    //==================================
    //Niter
    public void setNiter(int Ni){
	Niter = Ni;
    }

    public int getNiter(){
	return Niter;
    }
    //==================================
    //Nshow
    public void setNshow(int input){
	Nshow = input;
    }

    public int getNshow(){
	return Nshow;
    }
    //==================================
    //Iseed
    public void setIseed(int input){
	Iseed = input;
    }

    public void randomizeIseed(){
	Iseed = Math.abs(new java.util.Random().nextInt(0x7FFFFFFF));
    }

    public int getIseed(){
	return Iseed;
    }
    //==================================
    //AlphaA
    public void setAlphaA(double Aa){
	AlphaA = Aa;
    }

    public double getAlphaA(){
	return AlphaA;
    }
    //==================================
    //AlphaF
    public void setAlphaF(double Af){
	AlphaF = Af;
    }

    public double getAlphaF(){
	return AlphaF;
    }
    //==================================
    //linemeasure
    public void setLineMeasure(double lM){
	lineMeasure = lM;
    }

    public double getLineMeasure(){
	return lineMeasure;
    }
    //==================================
    //tag
    public void setTag(String t){
	tag = t;
    }

    public String getTag(){
	return tag;
    }
    //==================================
    //dir
    public void setDir(String input){
	dir = input;
    }

    public String getDir(){
	return dir;
    }
    //==================================
    //Nsol
    public void setNsol(int input){
	Nsol = input;
    }

    public int getNsol(){
	return Nsol;
    }
    //==================================
    //Na
    public void setNa(int input){
	Na = input;
    }

    public int getNa(){
	return Na;
    }
    //==================================
    //Nf
    public void setNf(int input){
	Nf = input;
    }

    public int getNf(){
	return Nf;
    }
    //==================================
    //ptwidth
    public void setPtwidth(double pw){
	ptwidth = pw;
    }

    public double getPtwidth(){
	return ptwidth;
    }
    //==================================
    //data
    public void setData(double[] d){
	data = d;
    }

    public double[] getData(){
	return data;
    }
    //==================================
    //acc
    public void setAcc(double[] input){
	acc = input;
    }

    public double[] getAcc(){
	return acc;
    }
    //==================================
    //additive
    public void setAdditive(double input){
	additive = input;
    }

    public double getAdditive(){
	return additive;
    }
    //==================================
    //multiplicative
    public void setMultiplicative(double input){
	multiplicative = input;
    }

    public double getMultiplicative(){
	return multiplicative;
    }
    //==================================
    //xLabels
    public void setXLabels(String[] xL){
	xLabels = xL;
    }

    public String[] getXLabels(){
	return xLabels;
    }
    //==================================
    //geneNames
    public void setGeneNames(String[] input){
	geneNames = input;
    }

    public String[] getGeneNames(){
	return geneNames;
    }
    //==================================
    //SPLIT_portion
    public void setSPLIT_portion(double S){
	SPLIT_portion = S;
    }

    public double getSPLIT_portion(){
	return SPLIT_portion;
    }
    //==================================
    //NfeetA
    public void setNfeetA(int NfA){
	NfeetA = NfA;
    }

    public int getNfeetA(){
	return NfeetA;
    }
    //==================================
    //NfootA
    public void setNfootA(int[] input){
	NfootA = input;
    }

    public int[] getNfootA(){
	return NfootA;
    }
    //==================================
    //NlistA
    public void setNlistA(int[][] NlA){
	NlistA = NlA;
    }

    public int[][] getNlistA(){
	return NlistA;
    }
    //==================================
    //Wtype
    public void setWtype(int Wt){
	Wtype = Wt;
    }

    public int getWtype(){
	return Wtype;
    }
    //==================================
    //Nfeet
    public void setNfeet(int Nfe){
	Nfeet = Nfe;
    }

    public int getNfeet(){
	return Nfeet;
    }
    //==================================
    //feetsizes
    public void setFeetsizes(int[] fs){
	feetsizes = fs;
    }

    public int[] getFeetsizes(){
	return feetsizes;
    }
    //==================================
    //uncertFlag
    public void setUncertFlag(int input){
	uncertFlag = input;
    }

    public int getUncertFlag(){
	return uncertFlag;
    }
    //==================================
    //StartRank
    public void setStartRank(int SR){
	StartRank = SR;
    }

    public int getStartRank(){
	return StartRank;
    }
    //==================================
    //EndRank
    public void setEndRank(int ER){
	EndRank = ER;
    }

    public int getEndRank(){
	return EndRank;
    }
    //==================================
    //Nchains
    public void setNchains(int Nc){
	Nchains = Nc;
    }

    public int getNchains(){
	return Nchains;
    }
}
