// Class: PattS
//
// Purpose: Server program for running PattRun with RMI. Mainly relays method calls
//          from PattServer to PattRun, but also opens textAreas to display outputs from 
//          different servers
//
// History: v0.1 8/03 WFS
//          v1.1 6/06 WFS
//
// COPYRIGHT, 2003, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// NERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package rmi;

import java.awt.BorderLayout;
import java.rmi.*;
import java.rmi.server.*;
import java.util.HashMap;
import java.util.Properties;
import javax.swing.*;

import pattviewer.*;
import lib.*;
import textfields.*;

public class PattS extends UnicastRemoteObject implements PattI {
    private static final long serialVersionUID = 200601l;
    public static int version = 200601;
    public static String copyright = "Copyright 2006, Fox Chase Cancer Center";

    PattI thisPatt;
    HashMap<String,JFrame> frames=new HashMap<String,JFrame>();

    /*
    * sets thisPatt to pattr
    */
    public PattS(PattI pattr) throws RemoteException{
	thisPatt=pattr;
    }

    /*
    * either finds the frame associated with this ip address or creates a 
    * new one and sends the C output to it.
    */
    public void guiPrint(String output) throws Exception{
	if(!frames.containsKey(getClientHost())){
	    JFrame outputs=new JFrame();
	    frames.put(getClientHost(),outputs);
	    CommandPane cPane = new CommandPane();
	    outputs.setSize(500,350);
	    outputs.getContentPane().setLayout(new BorderLayout());
	    outputs.getContentPane().add(/*jsp*/cPane,BorderLayout.CENTER);
	    try{
		outputs.setTitle(getClientHost());
	    }catch(Exception exception){}
	}
	JFrame outputs=(JFrame)frames.get(getClientHost());
	if(!outputs.isVisible()){outputs.setVisible(true);}
	((CommandPane)outputs.getContentPane().getComponent(0)).guiPrint(output);
    }

    /*
     * relays the writeGPE call to thisPatt
     */
    public void writePattOut(String filename, GenpatExp pattOut)throws RemoteException{
	thisPatt.writePattOut(filename,pattOut);
    }

    /*
     * returns the available algorithms which thisPatt can run
     */
    public Library[] getLibraries() throws RemoteException{
	return thisPatt.getLibraries();
    }

    /*
     * returns the status thisPatt
     */
    public String[] getStatus()throws RemoteException{
	return thisPatt.getStatus();
    }
    
    /*
     * runs cVersion[whichC] with input pattIn on port p in thisPatt
     */
    public void run(int whichC,PattInput pattIn,int p)throws RemoteException{
	thisPatt.run(whichC,pattIn,p);
    }
    
    /*
     * returns whether an algorithm is running on thisPatt
     */
    public boolean isBusy()throws RemoteException{
	return thisPatt.isBusy();
    }
}
