// Class: CommandPane
//
// Purpose: text area to display output for PattRun. Will scroll down when text is added if the
//          scroll bar is at the bottom.
//
// History v1.0 7/06 WFS
//
// COPYRIGHT, 2006, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package textfields;

import java.awt.BorderLayout;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;

import javax.swing.*;

public class CommandPane extends JPanel{
    boolean bottom;          // true if the scrollbar is at the bottom
    JTextArea displayText;   // text area
    JScrollBar vScroll;      // scrolling
    int oldMax;              // old maximum value for the JScrollBar.

    /*
     * creates the gui. It is a simple textArea with a scroll pane and an adjustment listener
     * which either stops at the current display or locks on to the bottom when new text is appended.
     */
    public CommandPane(){
	super();
	this.setLayout(new BorderLayout());

	displayText = new JTextArea();
	displayText.setEditable(false);
	displayText.setLineWrap(true);
	displayText.setWrapStyleWord(true);

	JScrollPane scroll = new JScrollPane(displayText);
	scroll.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	vScroll = scroll.getVerticalScrollBar();
	/*
	 * determines whether the viewport is at the bottom of the text area and stays at the bottom
	 * when text is added if so.
	 */
	vScroll.addAdjustmentListener(new AdjustmentListener(){
		public void adjustmentValueChanged(AdjustmentEvent e){
		    if(oldMax == vScroll.getMaximum()){  // this is true iff the user moves the scrollbar
			bottom = vScroll.getValue()+vScroll.getModel().getExtent()==vScroll.getMaximum();
		    }else if(bottom){            // scroll to the bottom if on the bottom before guiPrint
			vScroll.setValue(vScroll.getMaximum()-vScroll.getModel().getExtent());
		    }
		    oldMax = vScroll.getMaximum();
		}
	    });
	this.add(scroll,BorderLayout.CENTER);
	bottom = true;
	oldMax = vScroll.getMaximum();
    }

    /*
     * clears the text from the textArea
     */
    public void clear(){
	displayText.setText("");
    }

    /*
     * adds a line of text to the textArea
     */
    public void guiPrint(String text){
	displayText.append(text+"\n");
    }

    /*
     * convenience method calls toString method of the object for display
     */
    public void guiPrint(Object o){
	guiPrint(o.toString());
    }
}
