// Class: DataField
//
// Purpose: JTextField which executes a user-supplied action on ENTER keystroke
//
// History: v1.0 11/06 WFS
//
// COPYRIGHT, 2006, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program;  if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA   02111-1307, USA.

package textfields;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

public class DataField extends JTextField{
    private static final long serialVersionUID = 200601l;

    protected JTextField self = this;

    public DataField(AbstractAction action){
	super();
	init(action);
    }

    public DataField(int size, AbstractAction action){
	super(size);
	init(action);
    }

    public DataField(int size, String text, AbstractAction action){
	super(size);
	init(action);
	setText(text);
    }

    public DataField(int size, int text, AbstractAction action){
	super(size);
	init(action);
	restrict();
	setText(Integer.toString(text));
    }

    private void init(AbstractAction action){
	getInputMap().put(KeyStroke.getKeyStroke("ENTER"),"connect");
	/*
	 * connect to servers when a new port is entered and ENTER is pressed
	 */
	getActionMap().put("connect",action);

    }

    /*
     * restrict entries in portField to integers
     */
    private void restrict(){
	setDocument(new PlainDocument(){
	    private static final long serialVersionUID = 200601;
	    public void insertString(int offs, String str,
					AttributeSet a)throws BadLocationException{
		replace(offs,0,str,a);
	    }
	    public void remove(int offs, int len)throws BadLocationException{
		replace(offs,len,"",null);
	    }
	    public void replace(int offs, int len, String text,
					AttributeSet a)throws BadLocationException{
		String newText = self.getText().substring(0,offs)+text+
				self.getText().substring(offs+len);
		try{
		    if(!newText.equals("")){
			Integer.parseInt(newText);
		    }
		    super.remove(offs,len);
		    super.insertString(offs,text,a);
		    validate();
		}catch(NumberFormatException nfe){}
	    }
	});
    }
}