// Class: ControlAlpha
//
// Purpose: wrapper around a j3d alpha which adds reversing and setting functionality
//
// History v1.0 6/04 WFS
//
// COPYRIGHT, 2004, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program;  if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA   02111-1307, USA.

package viewer;

import javax.vecmath.*;
import javax.media.j3d.*;

public class ControlAlpha extends Alpha{
	Alpha alpha;
	private float isFlipped;
	private float scalar;

	public void flip(){
		float holder = scalar;
		scalar = 2f * value() - holder;
		isFlipped *=-1f;
	}

	public void scale(float zero){
		scalar+=zero-value();
	}

	public boolean isFlipped(){return (isFlipped==-1f);}

	public ControlAlpha(){
		super();
		alpha = new Alpha();
		scalar=0f;
		isFlipped=1f;
	}

	public float value(){
		float temp = scalar + isFlipped * alpha.value();
		if(temp<0f){
			scalar+=1f;
			temp+=1f;
		}else if(temp>1f){
			scalar-=1f;
			temp-=1f;
		}
		return temp;
	}

	public float value(long atTime){
		float temp = scalar + isFlipped * alpha.value(atTime);
		if(temp<0f){
			temp+=1f;
		}else if(temp>1f){
			temp-=1f;
		}
		return temp;
	}

	public void pause(){
		alpha.pause();
	}

	public void resume(){
		alpha.resume();
	}

	public void setMode(int mode){
		alpha.setMode(mode);
	}

	public void setIncreasingAlphaDuration(long increasingAlphaDuration){
		alpha.setIncreasingAlphaDuration(increasingAlphaDuration);
	}

	public void setDecreasingAlphaDuration(long decreasingAlphaDuration){
		alpha.setDecreasingAlphaDuration(decreasingAlphaDuration);
	}
}
