// Class: Graph3D
//
// Purpose: Creates a 3D image containing an axis and allows users to plot points
//
// History: v0.1 7/04 William F Speier IV
//
// COPYRIGHT, 2004, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program;  if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA   02111-1307, USA.

package viewer;

import javax.swing.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;

public class Graph3D extends JPanel{
    public static int version = 200401;
    public static String copyright = "Copyright 2004, Fox Chase Cancer Center";

    Canvas3D canvas;
    ControlAlpha lAlpha,uAlpha,vAlpha;
    MouseAction ma;
    TransformGroup pointTrans;
    String[] xLabels,yLabels,zLabels;
    boolean hasxLabels=false,hasyLabels=false,haszLabels=false,boundsSet=false,changeOnce=false;
    float xMin,yMin,zMin,xMax,yMax,zMax;
    Locale l;
    BranchGroup scene, viewGroup, labelBranch;
    PointArray pArray;
    boolean pointsAdded=false;
    Shape3D pointShape;
    boolean[] isCircled;
    CircledListener circledListener;
    Text2D textX, textY, textZ;

    public static void main(String[] args){
	JFrame frame = new JFrame();
	frame.setSize(400,400);
	frame.getContentPane().setLayout(new BorderLayout());
	Graph3D g3d = new Graph3D();
	frame.getContentPane().add(g3d,BorderLayout.CENTER);
	frame.setVisible(true);
	float[][] pts = {{0f,2f,3f},{0f,1f,3f},{2f,0f,2f},{1f,1f,0f}};
	float[][] pts2 = {{3f,3f,3f}};
	g3d.setBounds(0f,4f,-1f,5f,-1f,3f);
	g3d.addPoints(pts);
	g3d.addPoints(pts2,new Color3f(1f,0f,0f));
//	g3d.setLabels("XX","YY","ZZ");
//	g3d.unsetBounds();
    }

    public Graph3D(){
	super();
	this.setLayout(new BorderLayout());
	init();
    }
    private void init(){
	GraphicsConfigTemplate3D templ = new GraphicsConfigTemplate3D();
	GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
	GraphicsDevice device = env.getDefaultScreenDevice();
	GraphicsConfiguration config = device.getBestConfiguration(templ);
	canvas = new Canvas3D(config);
	this.add("Center",canvas);
	this.setVisible(true);
	VirtualUniverse vu = new VirtualUniverse();
	l = new Locale(vu);
	viewGroup = new BranchGroup();
	ViewPlatform vp = new ViewPlatform();
	View view = new View();
	view.addCanvas3D(canvas);
	view.setPhysicalBody(new PhysicalBody());
	view.setPhysicalEnvironment(new PhysicalEnvironment());
	view.attachViewPlatform(vp);

	//transform group which contains the viewing platform and moves it 
	TransformGroup View_TransformGroup = new TransformGroup();
	View_TransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	View_TransformGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	View_TransformGroup.addChild(vp);
	viewGroup.addChild(View_TransformGroup);

	// moves the camera back
	Transform3D View_Transform3D = new Transform3D();
	View_Transform3D.setTranslation(new Vector3f(0f,0f,30f));
	View_TransformGroup.setTransform(View_Transform3D);

	//transform group for the points which is used for scaling
	pointTrans = new TransformGroup();
	pointTrans.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
	pointTrans.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
	pointTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

	//keystrokes for rotation and zooming.  work with private class keyAction defined below
	//keystrokes are a higher level than key events which can be platform-dependant
	this.getInputMap().put(KeyStroke.getKeyStroke("pressed EQUALS"),"zoom in");
	this.getInputMap().put(KeyStroke.getKeyStroke("shift pressed EQUALS"),"zoom in");
	this.getActionMap().put("zoom in",new keyAction(0));
	this.getInputMap().put(KeyStroke.getKeyStroke("released EQUALS"),"stop zoom in");
	this.getInputMap().put(KeyStroke.getKeyStroke("shift released EQUALS"),"stop zoom in");
	this.getActionMap().put("stop zoom in",new keyAction(1));
	this.getInputMap().put(KeyStroke.getKeyStroke("pressed MINUS"),"zoom out");
	this.getActionMap().put("zoom out",new keyAction(2));
	this.getInputMap().put(KeyStroke.getKeyStroke("released MINUS"),"stop zoom out");
	this.getActionMap().put("stop zoom out",new keyAction(3));

	this.getInputMap().put(KeyStroke.getKeyStroke("pressed RIGHT"),"rot right");
	this.getActionMap().put("rot right",new keyAction(4));
	this.getInputMap().put(KeyStroke.getKeyStroke("released RIGHT"),"stop rot right");
	this.getActionMap().put("stop rot right",new keyAction(5));
	this.getInputMap().put(KeyStroke.getKeyStroke("pressed LEFT"),"rot left");
	this.getActionMap().put("rot left",new keyAction(6));
	this.getInputMap().put(KeyStroke.getKeyStroke("released LEFT"),"stop rot left");
	this.getActionMap().put("stop rot left",new keyAction(7));

	this.getInputMap().put(KeyStroke.getKeyStroke("pressed UP"),"rot up");
	this.getActionMap().put("rot up",new keyAction(8));
	this.getInputMap().put(KeyStroke.getKeyStroke("released UP"),"stop rot up");
	this.getActionMap().put("stop rot up",new keyAction(9));
	this.getInputMap().put(KeyStroke.getKeyStroke("pressed DOWN"),"rot down");
	this.getActionMap().put("rot down",new keyAction(10));
	this.getInputMap().put(KeyStroke.getKeyStroke("released DOWN"),"stop rot down");
	this.getActionMap().put("stop rot down",new keyAction(11));

	//sets the max and min so that adding any point will provide an ccurate max and min
	xMin = Float.MAX_VALUE;
	yMin = Float.MAX_VALUE;
	zMin = Float.MAX_VALUE;
	xMax = Float.MIN_VALUE;
	yMax = Float.MIN_VALUE;
	zMax = Float.MIN_VALUE;

	//branch group coontaining the axis labels which are set automatically or with setAxisLabels
	labelBranch = new BranchGroup();
	labelBranch.setCapability(labelBranch.ALLOW_CHILDREN_EXTEND);
	labelBranch.setCapability(labelBranch.ALLOW_CHILDREN_WRITE);

	scene = createSceneGraph(); //creates the scene branch containing all geometry
	viewGroup.compile(); //possibly not helpful because of the lack of geometry
	l.addBranchGraph(viewGroup);
	scene.compile();
	l.addBranchGraph(scene);
	isCircled = new boolean[0];
    }

    //provides an array of the indices of points circled in the graph
    //used with the circledEventListener to provide information for the circledEvent
    public int[] getCircledIndices(){
	int temp=0;
	for(int i=0;i<isCircled.length;i++){
		if(isCircled[i]){temp++;}
	}
	int[] circledIndices = new int[temp];
	int counter=0;
	for(int i=0;i<isCircled.length;i++){
		if(isCircled[i]){
			circledIndices[counter]=i;
			counter++;
		}
	}
	return circledIndices;
    }

    //adds points with the default color: white
    public void addPoints(float[][] dataPoints){
	addPoints(dataPoints,new Color3f(1f,1f,1f));
    }

    //sets the text of the labels for each axis located at the end of the line
    public void setLabels(String x, String y, String z){
	textX.setString(x);
	textY.setString(y);
	textZ.setString(z);
    }

    //adds points, the color attribute is not yet used
    //the new bounds are then found by getminmax
    public void addPoints(float[][] dataPoints, Color3f color){
	ma.reset();
	Point3f[] oldPoints=new Point3f[0];
	if(pointsAdded){
		oldPoints=new Point3f[pArray.getVertexCount()];
		for(int i = 0;i<oldPoints.length;i++){
			oldPoints[i] = new Point3f();
			pArray.getCoordinate(i,oldPoints[i]);
		}
	}
	pArray = new PointArray(dataPoints.length+oldPoints.length,PointArray.COORDINATES);
	pArray.setCapability(PointArray.ALLOW_COORDINATE_READ);
	pArray.setCapability(PointArray.ALLOW_COUNT_READ);
	pArray.setCoordinates(0,oldPoints);
	for(int i=0;i<dataPoints.length;i++){
	    pArray.setCoordinate(i+oldPoints.length,new Point3f(dataPoints[i]));
	}
	getMinMax(dataPoints);
	pointShape.setGeometry(pArray);
	pointsAdded=true;
    }

    //resets the graph by removing the points and resetting the bounds
    public void removePoints(){
	ma.reset();
	pointsAdded=false;
	pointShape.setGeometry(null);
	if(!boundsSet){pointTrans.setTransform(new Transform3D());}
	pArray = null;
	xMin = Float.MAX_VALUE;
	yMin = Float.MAX_VALUE;
	zMin = Float.MAX_VALUE;
	xMax = Float.MIN_VALUE;
	yMax = Float.MIN_VALUE;
	zMax = Float.MIN_VALUE;
    }

    // gets the bounds for each axis and then rescales the points, and changes
    // the labels accordingly if the bounds change
    private void getMinMax(float[][] dataPoints){
	boolean changed=false;
	float pxMin = xMin;
	float pyMin = yMin;
	float pzMin = zMin;
	float pxMax = xMax;
	float pyMax = yMax;
	float pzMax = zMax;
	for(int i = 0; i < dataPoints.length;i++){
		if(dataPoints[i][0]<=xMin){
			changed=true;
			xMin=dataPoints[i][0];
		}	
		if(dataPoints[i][0]>=xMax){
			changed=true;
			xMax=dataPoints[i][0];
		}	
		if(dataPoints[i][1]<=yMin){
			changed=true;
			yMin=dataPoints[i][1];
		}	
		if(dataPoints[i][1]>=yMax){
			changed=true;
			yMax=dataPoints[i][1];
		}	
		if(dataPoints[i][2]<=zMin){
			changed=true;
			zMin=dataPoints[i][2];
		}	
		if(dataPoints[i][2]>=zMax){
			changed=true;
			zMax=dataPoints[i][2];
		}
	}
	if((changed&&!boundsSet)||changeOnce){
		float[] trans ={10f/(xMax-xMin),0,		0,		10*-xMin/(xMax-xMin),
				0,		10f/(yMax-yMin),0,		10*-yMin/(yMax-yMin),
				0,		0,		10f/(zMax-zMin),10*-zMin/(zMax-zMin),
				0,		0,		0,		1};
		pointTrans.setTransform(new Transform3D(trans));
		setLabels();
		changeOnce=false;
	}
    }

    // sets the x axis labels to the strings in xLabs
    public void setxLabels(String[] xLabs){
	xLabels=xLabs;
	hasxLabels=true;
	addLabels(labelBranch);
    }

    // sets the y axis labels to the strings in yLabs
    public void setyLabels(String[] yLabs){
	yLabels=yLabs;
	hasyLabels=true;
	addLabels(labelBranch);
    }

    // sets the z axis labels to the strings in zLabs
    public void setzLabels(String[] zLabs){
	zLabels=zLabs;
	haszLabels=true;
	addLabels(labelBranch);
    }

    // resets the x axis labels to default
    public void removexLabels(){
	hasxLabels=false;
	setLabels();
    }

    // resets the y axis labels to default
    public void removeyLabels(){
	hasyLabels=false;
	setLabels();
    }

    // resets the z axis labels to default
    public void removeLabels(){
	haszLabels=false;
	setLabels();
    }

    // manually sets the bounds of the graph
    // the bounds of the graph will not be reset and points can occur outside of the bounds
    // the min and max values are still saved and will be used if the bounds are unset
    public void setBounds(float xi,float xa,float yi,float ya,float zi,float za){
	boundsSet = true;
	float[] holder = {xMin,xMax,yMin,yMax,zMin,zMax};
	xMin=xi;
	xMax=xa;
	yMin=yi;
	yMax=ya;
	zMin=zi;
	zMax=za;
	float[] trans ={10f/(xMax-xMin),0,		0,		10*-xMin/(xMax-xMin),
			0,		10f/(yMax-yMin),0,		10*-yMin/(yMax-yMin),
			0,		0,		10f/(zMax-zMin),10*-zMin/(zMax-zMin),
			0,		0,		0,		1};
	pointTrans.setTransform(new Transform3D(trans));
	setLabels();
	xMin=holder[0];
	xMax=holder[1];
	yMin=holder[2];
	yMax=holder[3];
	zMin=holder[4];
	zMax=holder[5];
    }

    // resets the bounds to the min and max values
    // the bounds will again be changed if the min and max values are changed
    public void unsetBounds(){
	boundsSet = false;
	changeOnce=true;
	getMinMax(new float[0][0]);
    }

    // if the axis labels have not been set, this sets the labels to the defaults
    // the defaults consist of 11 labels equally spaced and ranging form the min
    // to the max value for each axis
    // DecimalFormats are used to limit the number of figures shown.  It takes the
    // order of magnitude of the range and provides enough decimal places to show
    // that difference.  At least one decimal position is always shown if one exists
    private void setLabels(){
	if(!hasxLabels){
	    String pattern = new String("0.#");
		float range = xMax-xMin;
		int log = (int)(Math.log(range)/Math.log(10d));
		if(log>0d){
			for(int i=0;i<log;i++){
				pattern="#"+pattern;
			}
		}else{
			for(int i=0;i>=log;i--){
				pattern=pattern+"#";
			}
		}
	    DecimalFormat format = new DecimalFormat(pattern);
	    xLabels=new String[11];
	    float xinterval = 1f/(float)(xLabels.length-1);
	    for(int i = 0;i<xLabels.length;i++){
		xLabels[i]=format.format((xMax-xMin)*xinterval*(float)i+xMin);
	    }
	}
	if(!hasyLabels){
	    String pattern = new String("0.#");
		float range = yMax-yMin;
		int log = (int)(Math.log(range)/Math.log(10d));
		if(log>0d){
			for(int i=0;i<log;i++){
				pattern="#"+pattern;
			}
		}else{
			for(int i=0;i>=log;i--){
				pattern=pattern+"#";
			}
		}
	    DecimalFormat format = new DecimalFormat(pattern);
	    yLabels=new String[11];
	    float yinterval = 1f/(float)(yLabels.length-1);
	    for(int i = 0;i<yLabels.length;i++){
		yLabels[i]=format.format((yMax-yMin)*yinterval*(float)i+yMin);
	    }
	}
	if(!haszLabels){
	    String pattern = new String("0.#");
		float range = zMax-zMin;
		int log = (int)(Math.log(range)/Math.log(10d));
		if(log>0d){
			for(int i=0;i<log;i++){
				pattern="#"+pattern;
			}
		}else{
			for(int i=0;i>=log;i--){
				pattern=pattern+"#";
			}
		}
	    DecimalFormat format = new DecimalFormat(pattern);
	    zLabels=new String[11];
	    float zinterval = 1f/(float)(zLabels.length-1);
	    for(int i = 0;i<zLabels.length;i++){
		zLabels[i]=format.format((zMax-zMin)*zinterval*(float)i+zMin);
	    }
	}
	addLabels(labelBranch);
    }

    // adds the labels contained in the string arrays to the axis along with a line
    // from the point to which it corresponds.  The strings are displayed as Text2D
    // objects with a billboard object attatched.  Problems with Text2D exist which
    // can result in the loss of graphics and leave a white square.  This does not
    // occur often and can usually be fixed with a simple refresh.
    private void addLabels(Group graphBranch){
	graphBranch.removeAllChildren();
	BranchGroup temp = new BranchGroup();
	temp.setCapability(BranchGroup.ALLOW_DETACH);
	for(int i = 0;i<xLabels.length;i++){
		TransformGroup fontT = new TransformGroup();
		LineArray la = new LineArray(2,LineArray.COORDINATES);
		la.setCoordinate(0,new Point3f(0f,.3f,.3f));
		la.setCoordinate(1,new Point3f(0f,.5f,.5f));
		Shape3D laShape=new Shape3D(la);
		laShape.setPickable(false);
		fontT.addChild(laShape);
		temp.addChild(fontT);

		Transform3D translate = new Transform3D();
		translate.setTranslation(new Vector3f((float)i,-.5f,-.5f));
		fontT.setTransform(translate);

		TransformGroup billT = new TransformGroup();
		billT.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);		
		Text2D text = new Text2D(xLabels[i],new Color3f(1f,1f,1f),"Arial",100,Font.PLAIN);
		text.setPickable(false);
		billT.addChild(text);
		fontT.addChild(billT);

		Billboard billboard = new Billboard(billT,Billboard.ROTATE_ABOUT_POINT,new Point3f());
		billboard.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
		fontT.addChild(billboard);
	}
	for(int i = 0;i<zLabels.length;i++){
		TransformGroup fontT = new TransformGroup();
		LineArray la = new LineArray(2,LineArray.COORDINATES);
		la.setCoordinate(0,new Point3f(.3f,.3f,0f));
		la.setCoordinate(1,new Point3f(.5f,.5f,0f));
		Shape3D laShape=new Shape3D(la);
		laShape.setPickable(false);
		fontT.addChild(laShape);
		temp.addChild(fontT);

		Transform3D translate = new Transform3D();
		translate.setTranslation(new Vector3f(-.5f,-.5f,(float)i));
		fontT.setTransform(translate);

		TransformGroup billT = new TransformGroup();
		billT.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		Text2D text = new Text2D(zLabels[i],new Color3f(1f,1f,1f),"Arial",100,Font.PLAIN);
		text.setPickable(false);
		billT.addChild(text);
		fontT.addChild(billT);

		Billboard billboard = new Billboard(billT,Billboard.ROTATE_ABOUT_POINT,new Point3f());
		billboard.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
		fontT.addChild(billboard);
	}
	for(int i = 0;i<yLabels.length;i++){
		TransformGroup fontT = new TransformGroup();
		LineArray la = new LineArray(2,LineArray.COORDINATES);
		la.setCoordinate(0,new Point3f(.3f,.2f,.3f));
		la.setCoordinate(1,new Point3f(.5f,.2f,.5f));
		Shape3D laShape=new Shape3D(la);
		laShape.setPickable(false);
		fontT.addChild(laShape);
		temp.addChild(fontT);

		Transform3D translate = new Transform3D();
		translate.setTranslation(new Vector3f(-.5f,(float)i-.2f,-.5f));
		fontT.setTransform(translate);

		TransformGroup billT = new TransformGroup();
		billT.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		Text2D text = new Text2D(yLabels[i],new Color3f(1f,1f,1f),"Arial",100,Font.PLAIN);
		text.setPickable(false);
		billT.addChild(text);
		fontT.addChild(billT);

		Billboard billboard = new Billboard(billT,Billboard.ROTATE_ABOUT_POINT,new Point3f());
		billboard.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
		fontT.addChild(billboard);
	}

	temp.compile();
	graphBranch.addChild(temp);
    }

    // resets the ControlAlphas to their default values and pauses them
    public void reset(){
	uAlpha.pause();
	lAlpha.pause();
	vAlpha.pause();
	uAlpha.scale(.5f);
	lAlpha.scale(0f);
	vAlpha.scale(0f);
    }

    // creates scene, the BranchGroup which contains all of the geometry
    private BranchGroup createSceneGraph(){
	BranchGroup objRoot = new BranchGroup();  // main brnchGroup which will be returned
	objRoot.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND); //allows labels to be added later
	TransformGroup graphBranch = new TransformGroup();
	graphBranch.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

	pointShape = new Shape3D(); // initializes the shape which contains the plotted points
	pointShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
	pointShape.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
	pointShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
	pointShape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
	pointShape.setCapability(Shape3D.ALLOW_LOCAL_TO_VWORLD_READ);

	BranchGroup pointBranch = new BranchGroup();
	pointBranch.addChild(pointShape);
	pointTrans.addChild(pointBranch);

	// creates a simple 3D axis 10x10x10 meters
	Appearance axisAppearance = new Appearance();
	axisAppearance.setColoringAttributes(new ColoringAttributes(new Color3f(1f,1f,1f),1));
	LineArray axis = new LineArray(6,LineArray.COORDINATES);
	double[] cVals={0d,0d,0d,
		10d,0d,0d,
		0d,0d,0d,
		0d,10d,0d,
		0d,0d,0d,
		0d,0d,10d};
	axis.setCoordinates(0,cVals);
	Shape3D axisShape = new Shape3D(axis,axisAppearance);
	axisShape.setPickable(false);

	graphBranch.addChild(axisShape);

	TransformGroup fontX = new TransformGroup();
	graphBranch.addChild(fontX);
	Transform3D translateX = new Transform3D();
	translateX.setTranslation(new Vector3f(11f,0f,0f));
	fontX.setTransform(translateX);
	TransformGroup billX = new TransformGroup();
	billX.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);		
	textX = new Text2D("X",new Color3f(1f,1f,1f),"Arial",100,Font.PLAIN);
	textX.setCapability(Text2D.ALLOW_APPEARANCE_READ);
	textX.getAppearance().setCapability(Appearance.ALLOW_TEXTURE_READ);
	textX.getAppearance().setCapability(Appearance.ALLOW_TEXTURE_WRITE);
	textX.setPickable(false);
	billX.addChild(textX);
	fontX.addChild(billX);
	Billboard billboardX = new Billboard(billX,Billboard.ROTATE_ABOUT_POINT,new Point3f());
	billboardX.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
	fontX.addChild(billboardX);

	TransformGroup fontY = new TransformGroup();
	graphBranch.addChild(fontY);
	Transform3D translateY = new Transform3D();
	translateY.setTranslation(new Vector3f(0f,11f,0f));
	fontY.setTransform(translateY);
	TransformGroup billY = new TransformGroup();
	billY.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);		
	textY = new Text2D("Y",new Color3f(1f,1f,1f),"Arial",100,Font.PLAIN);
	textY.setCapability(Text2D.ALLOW_APPEARANCE_READ);
	textY.getAppearance().setCapability(Appearance.ALLOW_TEXTURE_READ);
	textY.getAppearance().setCapability(Appearance.ALLOW_TEXTURE_WRITE);
	textY.setPickable(false);
	billY.addChild(textY);
	fontY.addChild(billY);
	Billboard billboardY = new Billboard(billY,Billboard.ROTATE_ABOUT_POINT,new Point3f());
	billboardY.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
	fontY.addChild(billboardY);

	TransformGroup fontZ = new TransformGroup();
	graphBranch.addChild(fontZ);
	Transform3D translateZ = new Transform3D();
	translateZ.setTranslation(new Vector3f(0f,0f,11f));
	fontZ.setTransform(translateZ);
	TransformGroup billZ = new TransformGroup();
	billZ.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);		
	textZ = new Text2D("Z",new Color3f(1f,1f,1f),"Arial",100,Font.PLAIN);
	textZ.setCapability(Text2D.ALLOW_APPEARANCE_READ);
	textZ.getAppearance().setCapability(Appearance.ALLOW_TEXTURE_READ);
	textZ.getAppearance().setCapability(Appearance.ALLOW_TEXTURE_WRITE);
	textZ.setPickable(false);
	billZ.addChild(textZ);
	fontZ.addChild(billZ);
	Billboard billboardZ = new Billboard(billZ,Billboard.ROTATE_ABOUT_POINT,new Point3f());
	billboardZ.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
	fontZ.addChild(billboardZ);

	TransformGroup labels = new TransformGroup();

	graphBranch.addChild(labelBranch);

	graphBranch.addChild(pointTrans);
	ma = new MouseAction(objRoot);
	canvas.addMouseMotionListener(ma);
	canvas.addMouseListener(ma);

	// moves the geometry of the graph so it is rotated about its center rather than the origin
	Transform3D center = new Transform3D();
	Transform3D centerRotate = new Transform3D();
	centerRotate.rotY(7d*Math.PI/4d);
	center.setTranslation(new Vector3f(-5f,-5f,-5f));
	centerRotate.mul(center);
	graphBranch.setTransform(centerRotate);

	// adds the zooming capability
	// the graph starts in the middle and can be moved along a line that extends 50 meters
	// foreward and backward from its origin.  It loops if it is moved past its bounds.
	TransformGroup uGroup = new TransformGroup();
	uGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	uAlpha = new ControlAlpha();
	uAlpha.setIncreasingAlphaDuration(5000);
	Transform3D axisTrans = new Transform3D();
	axisTrans.rotY(Math.PI/2d);
	PositionInterpolator uint = new PositionInterpolator(uAlpha,uGroup,axisTrans,-50,50);
	uint.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
	uGroup.addChild(uint);
	objRoot.addChild(uGroup);

	// rotates the graph about vertically about an axis that remains parallel to the screen
	TransformGroup vGroup = new TransformGroup();
	Transform3D vt = new Transform3D();
	vt.rotY(Math.PI/2f);
	Transform3D vt2 = new Transform3D();
	vt2.rotX(Math.PI/2f);
	vt.mul(vt2);
	vGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	vAlpha = new ControlAlpha();
	vAlpha.setIncreasingAlphaDuration(5000);
	RotationInterpolator vint = new RotationInterpolator(vAlpha,vGroup);
	vint.setTransformAxis(vt);
	vint.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
	vGroup.addChild(vint);
	uGroup.addChild(vGroup);

	//rotates the graph about its center horizontally
	TransformGroup lGroup = new TransformGroup();
	lGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	lAlpha = new ControlAlpha();
	lAlpha.setIncreasingAlphaDuration(5000);
	RotationInterpolator lint = new RotationInterpolator(lAlpha,lGroup);
	lint.setSchedulingBounds(new BoundingSphere(new Point3d(),1000.0));
	lGroup.addChild(lint);
	vGroup.addChild(lGroup);
	lGroup.addChild(graphBranch);

	reset();

	return objRoot;
    }

    // private class which contains the response to key presses
    private class keyAction extends AbstractAction{
	int direction; // integer which signals actionPerformed which key was pressed
	public keyAction(int d){
		direction=d;
	}
	public void actionPerformed(ActionEvent e){
	    ma.reset();
	    if(direction==0){ // equals pressed
		if(!uAlpha.isFlipped()){uAlpha.flip();}
		uAlpha.resume();
	    }
	    if(direction==1){ // equals released
		if(uAlpha.isFlipped()){uAlpha.pause();}
	    }
	    if(direction==2){ // minus pressed
		if(uAlpha.isFlipped()){uAlpha.flip();}
		uAlpha.resume();
	    }
	    if(direction==3){ // minus released
		if(!uAlpha.isFlipped()){uAlpha.pause();}
	    }
	    if(direction==4){ // right pressed
		if(!lAlpha.isFlipped()){lAlpha.flip();}
			lAlpha.resume();
	    }
	    if(direction==5){ // right released
		if(lAlpha.isFlipped()){lAlpha.pause();}
	    }
	    if(direction==6){ // left pressed
		if(lAlpha.isFlipped()){lAlpha.flip();}
		lAlpha.resume();
	    }
	    if(direction==7){ // left released
		if(!lAlpha.isFlipped()){lAlpha.pause();}
	    }
	    if(direction==8){ // up pressed
		if(!vAlpha.isFlipped()){vAlpha.flip();}
		vAlpha.resume();
	    }
	    if(direction==9){ // up released
		if(vAlpha.isFlipped()){vAlpha.pause();}
	    }
	    if(direction==10){ // down pressed
		if(vAlpha.isFlipped()){vAlpha.flip();}
		vAlpha.resume();
	    }
	    if(direction==11){ // down released
		if(!vAlpha.isFlipped()){vAlpha.pause();}
	    }
	}
    }

    // sets a listener for circling usually in the implementing class
    public void setCircledListener(CircledListener newListener){
	circledListener = newListener;
    }

    // catches and determines responses for mouse action events
    private class MouseAction extends MouseMotionAdapter implements MouseListener{
	BranchGroup branch;
	Shape3D picked;
	Appearance oldAppearance;
	BranchGroup circleBranch;
	Shape3D circle;
	Point3d initial,fin;
	
	// constructor creates the geometry for the circles
	public MouseAction(BranchGroup bg){
	    branch=bg;
	    circleBranch=new BranchGroup();
	    QuadArray quad = new QuadArray(4,QuadArray.COORDINATES|QuadArray.TEXTURE_COORDINATE_2);
	    quad.setCapability(QuadArray.ALLOW_COORDINATE_WRITE);
	    quad.setTextureCoordinate(0,0,new TexCoord2f(0f,1f));
	    quad.setTextureCoordinate(0,1,new TexCoord2f(0f,0f));
	    quad.setTextureCoordinate(0,2,new TexCoord2f(1f,0f));
	    quad.setTextureCoordinate(0,3,new TexCoord2f(1f,1f));
	    circle = new Shape3D(quad,createImage(128));
	    circle.setPickable(false);
	    circle.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
	    circle.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
	    circleBranch.addChild(circle);
	    branch.addChild(circleBranch);
	    initial = new Point3d();
	    fin = new Point3d();
	    
	    QuadArray scene = new QuadArray(4,QuadArray.COORDINATES);
	    scene.setCoordinate(0,new Point3f());
	    scene.setCoordinate(1,new Point3f());
	    scene.setCoordinate(2,new Point3f());
	    scene.setCoordinate(3,new Point3f());
	    scene.setCapability(QuadArray.ALLOW_COORDINATE_WRITE);
	}

	// creates the geometry for the circles.  used in the constructor
	public Appearance createImage(int height){
	    BufferedImage image = new BufferedImage(height,height,BufferedImage.TYPE_INT_ARGB);
	    Graphics2D g = image.createGraphics();
	    g.setColor(new Color(0,0,0,0));
	    g.clearRect(0,0,height,height);
	    g.setColor(new Color(1f,1f,1f,1f));
	    g.drawOval(1,1,height-2,height-2);
	    g.dispose();
	    ImageComponent2D ic = new ImageComponent2D(ImageComponent2D.FORMAT_RGBA,image);
	    Texture2D texture = new Texture2D(Texture2D.BASE_LEVEL,Texture.RGBA,height,height);
	    texture.setImage(0,ic);
	    texture.setEnable(true);
	    Appearance app = new Appearance();
	    app.setTexture(texture);
	    RenderingAttributes ra = new RenderingAttributes(false,false,0f,RenderingAttributes.GREATER);
	    app.setRenderingAttributes(ra);
	    app.setPolygonAttributes(new PolygonAttributes(PolygonAttributes.POLYGON_FILL,
							   PolygonAttributes.CULL_NONE,0f));
	    return app;
	}

	// saves the initial point
	public void mousePressed(MouseEvent e){
	    int x = e.getX();
	    int y = e.getY();
	    canvas.getPixelLocationInImagePlate(x,y,initial);
	}

	// resizes the circle object to extend to the current cursor position
	public void mouseDragged(MouseEvent e){
	    int x = e.getX();
	    int y = e.getY();
	    
	    canvas.getPixelLocationInImagePlate(x,y,fin);
	    Point3d v1 = new Point3d(initial.x,initial.y,initial.z);
	    Point3d v2 = new Point3d(initial.x,fin.y,initial.z);
	    Point3d v3 = new Point3d(fin.x,fin.y,initial.z);
	    Point3d v4 = new Point3d(fin.x,initial.y,initial.z);
	    Transform3D motion = new Transform3D();
	    canvas.getImagePlateToVworld(motion);
	    motion.transform(v1);
	    motion.transform(v2);
	    motion.transform(v3);
	    motion.transform(v4);
	    QuadArray quad = (QuadArray)circle.getGeometry();
	    quad.setCoordinate(0,v1);
	    quad.setCoordinate(1,v2);
	    quad.setCoordinate(2,v3);
	    quad.setCoordinate(3,v4);
	    circle.setGeometry(quad);
	}

	// gets the final position, finds all of the points in the circle by testing the
	// angle from the center to the angle of the bounds of the circle and then fires
	// a CircledEvent
	public void mouseReleased(MouseEvent e){
	    Point3d eyePos = new Point3d();
	    canvas.getCenterEyeInImagePlate(eyePos);
	    Transform3D motion = new Transform3D();
	    canvas.getImagePlateToVworld(motion);
	    Point3d center = new Point3d(.5f*(initial.x+fin.x),
					 .5f*(initial.y+fin.y),
					 .5f*(initial.z+fin.z));
	    motion.transform(eyePos);
	    motion.transform(center);
	    Vector3d centerVect = new Vector3d(center);
	    centerVect.sub(eyePos);
	    motion.transform(fin);
	    Vector3d limitVect = new Vector3d(fin);
	    limitVect.sub(eyePos);
	    double angleLimit = limitVect.angle(centerVect);
	    
	    double a2 = Math.pow((centerVect.x-limitVect.x),2d);
	    double b2 = Math.pow((centerVect.y-limitVect.y),2d);
	    
	    if(pArray!=null){
		Transform3D pointTransform = new Transform3D();
		pointShape.getLocalToVworld(pointTransform);
		Point3f point=new Point3f();
		isCircled = new boolean[pArray.getVertexCount()];
		boolean fire = false;
		for(int i = 0;i<isCircled.length;i++){
		    pArray.getCoordinate(i,point);
		    pointTransform.transform(point);
		    Vector3d vect = new Vector3d(point);
		    vect.sub(eyePos);
		    isCircled[i]=false;
		    
		    double n = vect.z/centerVect.z;
		    double n2 = Math.pow(n,2d);
		    if((Math.pow(vect.x-centerVect.x*n,2d))/(n2*a2)+(Math.pow(vect.y-centerVect.y*n,2d))/(n2*b2)<1d){	
			isCircled[i]=true;
			fire = true;
		    }
		}
		if(fire){
		    fireCircledEvent(getCircledIndices());
		}
	    }
	}

	// calls the circled event in the added CircledListener
	public void fireCircledEvent(int[] indices){
	    try{
		circledListener.circled(new CircledEvent(this, indices));
	    }catch(NullPointerException e){}
	}

	// erases the circle.  Used when the graph is rotated
	protected void reset(){
	    QuadArray quad = (QuadArray)circle.getGeometry();
	    quad.setCoordinate(0,new Point3f());
	    quad.setCoordinate(1,new Point3f());
	    quad.setCoordinate(2,new Point3f());
	    quad.setCoordinate(3,new Point3f());
	    circle.setGeometry(quad);
	}

	public void mouseClicked(MouseEvent e){}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e){}
	public void mouseMoved(MouseEvent e){}
    }
}
