// Class: PattFrame
//
// Purpose: Imports Bayesian Decomposition data, creates two and three-dimensional graphs, and exports data
//
// History: v1.0 7/03 WFS
//          v1.1 6/06 WFS
//
// COPYRIGHT, 2003, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

package viewer;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.ObjectInputStream;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Vector;

import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.standard.*;
import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

import pattviewer.GenpatExp;
import viewer.csi.gui.ImageCanvas;
import textfields.ImageFilter;

public class PattFrame extends JFrame {
    public static int version = 2002011;
    public static String copyright = "Copyright 2002, Fox Chase Cancer Center";

    String filename;
    ArrayList<float[][]> dataArray=new ArrayList<float[][]>(8);
    int Na,Nf,Nsol;
    String[] geneNames;
    static String[] graphNames={"Mock","Data","Residuals","Pattern","Distribution","Noise","Pattern SD","Distribution SD","Basis Vector","Circled Points"};
    String date;
    String[] xLabels;

    static String NOT_SELECTED="randy";
    int gene=0;
    int chiSq;
    int[] dataLabels=new int[5];
    boolean isBV=false,isCol=false;
    String[] BVlabel;

    JFileChooser jfcin = new JFileChooser();
    JFileChooser jfcout = new JFileChooser();
    JPanel savePanel=new JPanel();
    JCheckBox[] saveOptions=new JCheckBox[10];
    ComboBoxModel model;

    JPanel graphPanel = new JPanel();
    JButton load = new JButton("Load");
    public JButton right = new JButton(">"),left = new JButton("<");
    JTextField label = new JTextField();
    JButton bVect = new JButton("Basis Vector");
    JButton export = new JButton("Export");
    JButton print = new JButton("Print");
    JButton graph3d = new JButton("3D graph");

    JPanel graphChooser = new JPanel();
    JCheckBox[] buttonArray=new JCheckBox[5];
    JButton graphChoice = new JButton("Graph");
    Vector<String> vect1=new Vector<String>(2);
    JComboBox colRow; 

    ImageCanvas ic = new ImageCanvas();

    JFrame basisVect=new JFrame();
    JCheckBox[] radButtonArray;
    JComboBox[] comboBoxArray;
    JTextField[] textFieldArray;
    Vector<String> bvVector=new Vector<String>(2);
    JPanel bvectChoice = new JPanel();
    JButton graphLeft=new JButton("Graph");
    JPanel jPanel4 = new JPanel();

    Graph3D g3d;
    JFrame frame3D = new JFrame();
    JPanel panel3D = new JPanel();
    JLabel[] iLabs = new JLabel[3];
    JTextField[] indices = new JTextField[3];
    boolean has3d=true, is3d=false;
    JPanel lp = new JPanel();
    CardLayout cl = new CardLayout();
    int[] index;

    ActionListener listener;

    public PattFrame() {
	try {
	    pattInit();
	}catch(Exception exception) {
	    exception.printStackTrace();
	}
    }

    /*
    * sets all of the swing components to their initial states
    */
    private void pattInit() throws Exception {
	lp.setVisible(false);
	this.setTitle("No File Loaded");
	bvVector.add(new String(">"));
	bvVector.add(new String("<"));

	vect1.add(new String("row"));
	vect1.add(new String("col"));
	model=new DefaultComboBoxModel(vect1);
	colRow=new JComboBox(model);
	colRow.setSize(75,25);

	this.setSize(750,500);
	this.setDefaultCloseOperation(2);
	this.getContentPane().setLayout(new BorderLayout());
	graphPanel.setBorder(BorderFactory.createRaisedBevelBorder());
	graphPanel.setPreferredSize(new Dimension(325, 50));
	export.setVisible(false);
	print.setVisible(false);
	graph3d.setVisible(false);
	load.setPreferredSize(new Dimension(75, 25));
	jfcin.setFileFilter(new ImageFilter("gpe"));
	jfcin.setDialogTitle("choose a .gpe file");

	//action listener used by all buttons which calls the appropriate method
	listener=(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if(e.getActionCommand().equals("Label")){
			    label_actionPerformed(e);
			}else if(e.getActionCommand().equals(right.getActionCommand())){
			    rl_actionPerformed(true); 
			}else if(e.getActionCommand().equals(left.getActionCommand())){
			    rl_actionPerformed(false); 
			}else if(e.getActionCommand().equals(graphChoice.getActionCommand())){
			    graphChoice_actionPerformed(e);
			}else if(e.getActionCommand().equals("bvGraph")){
			    bvGraph_actionPerformed(e);
			}else if(e.getActionCommand().equals(bVect.getActionCommand())){
			    bVect_actionPerformed(e);
			}else if(e.getActionCommand().equals(load.getActionCommand())){
			    load_actionPerformed(e);
		 	}else if(e.getActionCommand().equals(export.getActionCommand())){
			    export_actionPerformed(e);
			}else if(e.getActionCommand().equals(print.getActionCommand())){
			    print_actionPerformed(e);
			}else if(e.getActionCommand().equals(graph3d.getActionCommand())){
			    graph3d_actionPerformed(e);
			}else if(e.getActionCommand().equals("buttonArray")){
			    Checkbox_actionPerformed(e);
			}else if(e.getActionCommand().equals("radButton")){
			    tempButton_actionPerformed(e);
			}else if(e.getActionCommand().equals("Plot")){
			    plot_actionPerformed(e);
			}
		}
	    });
	export.addActionListener(listener);
	print.addActionListener(listener);
	graph3d.addActionListener(listener);
	load.addActionListener(listener);
	left.addActionListener(listener);
	right.addActionListener(listener);
	label.addActionListener(listener);
	label.setActionCommand("Label");
	graphLeft.setActionCommand("bvGraph");

	left.setVisible(false);
	left.setPreferredSize(new Dimension(55, 40));
	right.setVisible(false);
	right.setPreferredSize(new Dimension(55, 40));
	label.setVisible(false);
	label.setPreferredSize(new Dimension(35,20));
	
	for(int i=0;i<graphNames.length-5;i++){
	    buttonArray[i]=new JCheckBox();
	    buttonArray[i].setText(graphNames[i]);
	    buttonArray[i].setActionCommand("buttonArray");
	    buttonArray[i].addActionListener(listener);
	}
	for(int i=0;i<graphNames.length;i++){
	    saveOptions[i]=new JCheckBox();
	    saveOptions[i].setText(graphNames[i]);
	}
	savePanel.setPreferredSize(new Dimension(135,120));
	jfcout.setPreferredSize(new Dimension(550,420));
	graphChoice.addActionListener(listener);
	graphChooser.setBorder(BorderFactory.createRaisedBevelBorder());
	graphChooser.setMinimumSize(new Dimension(300, 35));
	graphChooser.setPreferredSize(new Dimension(325, 50));

	frame3D.setResizable(false);
	frame3D.setTitle("A matrix index");
	frame3D.setSize(300,200);
	frame3D.getContentPane().add(panel3D,BorderLayout.CENTER);
	iLabs[0] = new JLabel("X axis");
	iLabs[1] = new JLabel("Y axis");
	iLabs[2] = new JLabel("Z axis");
	for(int i = 0;i<3;i++){
		JPanel temp = new JPanel();
		temp.setSize(300,30);
		indices[i] = new JTextField(Integer.toString(i),2);
		final JTextField tempText = indices[i];
		indices[i].setDocument(new PlainDocument(){
			private static final long serialVersionUID = 200601l;
			public void insertString(int offs, String str, AttributeSet a)throws BadLocationException{
			    replace(offs,0,str,a);
			}
			public void remove(int offs, int len)throws BadLocationException{
			    replace(offs,len,"",null);
			}
			public void replace(int offs,int len,String text,AttributeSet a)throws BadLocationException{
			    String s = tempText.getText();
			    String newText = s.substring(0,offs)+text+s.substring(offs+len);
			    if(newText.equals("")){
				super.remove(offs,len);
				super.insertString(offs,text,a);
			    }else {
				try{
				    int d = Integer.parseInt(newText);
				    super.remove(offs,len);
				    super.insertString(offs,text,a);
				}catch(Exception e){}				
			    }
			}
		    });
		temp.add(iLabs[i]);
		temp.add(indices[i]);
		panel3D.add(temp);
	}
	JPanel plotPanel = new JPanel();
	frame3D.getContentPane().add(plotPanel,BorderLayout.SOUTH);
	JButton plot = new JButton("Plot");
	plot.setActionCommand("Plot");
	plot.addActionListener(listener);
	plotPanel.add(plot);

	basisVect.setResizable(false);
	basisVect.setTitle("Basis Vector");
	bvectChoice.setBorder(BorderFactory.createRaisedBevelBorder());
	bvectChoice.setPreferredSize(new Dimension(300, 45));
	graphLeft.addActionListener(listener);
	//jPanel4.setBorder(BorderFactory.createRaisedBevelBorder());
	bVect.setVisible(false);
	bVect.addActionListener(listener);
	this.getContentPane().add(graphPanel, BorderLayout.SOUTH);

	jfcout.setAccessory(savePanel);
	for(int i=0;i<saveOptions.length;i++){  //adding all of the swing components
	    savePanel.add(saveOptions[i],null);
	    (saveOptions[i]).setPreferredSize(new Dimension(135,20));
	}
	graphPanel.add(left, null);
	graphPanel.add(label, null);
	graphPanel.add(right, null);
	graphPanel.add(load, null);
	graphPanel.add(bVect, null);
	graphPanel.add(export, null);
	graphPanel.add(print, null);
	graphPanel.add(graph3d, null);
	for(int i=0;i<buttonArray.length;i++){
	    graphChooser.add(buttonArray[i], null);
	}
	graphChooser.add(graphChoice, null);
	graphChooser.add(colRow, null);
	basisVect.getContentPane().add(bvectChoice, BorderLayout.SOUTH);
	bvectChoice.add(graphLeft, BorderLayout.CENTER);
	basisVect.getContentPane().add(jPanel4, BorderLayout.CENTER);

	lp.setLayout(cl);
	try{
		g3d=new Graph3D();
		g3d.setCircledListener(new CircledListener() {
			public void circled(CircledEvent e) {
				g3d_circled(e);
			}
		});
		lp.add(g3d,"g3d");
		cl.addLayoutComponent(g3d,new String("g3d"));
	}catch(NoClassDefFoundError exception){
		has3d=false;
	}
	lp.add(ic,"ic");
	cl.addLayoutComponent(ic,new String("ic"));
	this.getContentPane().add(lp,BorderLayout.CENTER);
    }

    private void g3d_circled(CircledEvent e){
	is3d=true;
	JFrame circled = new JFrame();
	circled.setSize(new Dimension(200,200));
	circled.getContentPane().setLayout(new BorderLayout());
	JScrollPane scroll = new JScrollPane();
	circled.getContentPane().add(scroll);
	JTextPane circledText = new JTextPane();
	scroll.getViewport().add(circledText);
	index = e.getIndices();
	String names = new String("");
	circled.setVisible(true);
	for(int i=0;i<index.length;i++){
		if(geneNames!=null&&geneNames.length==Na){
			names+=geneNames[index[i]]+"\n";
		}else{
			names+=index[i]+"\n";
		}
	}
	circledText.setText(names);
	circledText.setEditable(false);
    }

    /*
    * displays the filechooser and gets the GenpatExp from the selected file
    */
    private void load_actionPerformed(ActionEvent e) {
	if(jfcin.showOpenDialog(this) == JFileChooser.APPROVE_OPTION){
	    File myData = jfcin.getSelectedFile();
	    filename =myData.getName();
		
	    try{ 
		String filePath=myData.getCanonicalPath();
		FileInputStream outfips = new FileInputStream(myData);
		BufferedInputStream outisr = new BufferedInputStream(outfips);
		ObjectInputStream outbr = new ObjectInputStream(outisr);
		    
		GenpatExp gpe = (GenpatExp) outbr.readObject();

		outfips.close();
		outisr.close();
		outbr.close();
		try{
		    Na=gpe.getNa();
		    BufferedReader br = new BufferedReader(new FileReader(filePath.replaceAll(".gpe",".gnm")));
		    geneNames=new String[Na];
		    for(int i=0;i<Na;i++){
			geneNames[i]=br.readLine();
		    }
		    br.close();
		}catch(FileNotFoundException fnfe){
		    geneNames=null;
		}
		setData(gpe,geneNames);
	    }catch (Exception exception){
		JOptionPane.showMessageDialog(this,"Incorrect File format","WARNING",JOptionPane.WARNING_MESSAGE);
	    }
	}
    }

    /*
     * converts a double array into a float array because the GenpatExp object contains doubles, but
     * the graphing program requires floats.
     */
    private float[][] convert(double[][] d){
	float[][] f = new float[d.length][d[0].length];
	for(int i=0;i<f.length;i++){
	    for(int j=0;j<f[i].length;j++){
		f[i][j] = (float)d[i][j];
	    }
	}
	return f;
    }

    /*
    * gets the data from gpe
    */
    public void setData(GenpatExp gpe,String[] gn)throws NullPointerException{
	if(gpe.getData().length==0){
	    throw new NullPointerException("no data in GenpatExp");
	}
	//index=new int[0];
	lp.setVisible(true);
	cl.show(lp,"ic");

	xLabels=gpe.getXLabels();
	geneNames=gn;
	dataArray.clear();
	dataArray.add(0,convert(gpe.getMock()));
	dataArray.add(1,convert(gpe.getData()));
 	Na = gpe.getNa();
	Nf = gpe.getNf();
	Nsol = gpe.getNsol();
	float[][] tempF = new float[Na][Nf];
	for (int j = 0; j < Na; j++){
	    for(int k = 0; k < Nf; k++){
		tempF[j][k] = ((float[][])dataArray.get(1))[j][k]-((float[][])dataArray.get(0))[j][k];
	    }
	}
	dataArray.add(2,tempF);
	dataArray.add(3,convert(gpe.getPattern()));
	dataArray.add(4,convert(gpe.getDistribution()));
	dataArray.add(5,convert(gpe.getNoise()));
	dataArray.add(6,convert(gpe.getPattSD()));
	dataArray.add(7,convert(gpe.getDistSD()));
	date=gpe.getDate();
	    
	left.setVisible(false);
	right.setVisible(false);
	label.setVisible(false);

	chiSq=(int)computeChiSquared();
	filename=gpe.getTag().replaceAll(".inf",".gpe");
	this.setTitle(filename+" : Chi Sq = "+Integer.toString(chiSq));
  
	bVect.setVisible(true);
	export.setVisible(true);
	print.setVisible(false);
	graph3d.setVisible(has3d);

	basisVect.setSize(300,(110+40*Nsol));
	radButtonArray=new JCheckBox[Nsol];
	textFieldArray=new JTextField[Nsol];
	comboBoxArray=new JComboBox[Nsol];
	jPanel4.removeAll();
	for(int i=0;i<Nsol;i++){
		radButtonArray[i]=new JCheckBox();
		radButtonArray[i].setPreferredSize(new Dimension(135, 25));
		radButtonArray[i].addActionListener(listener);
		radButtonArray[i].setActionCommand("radButton");
		radButtonArray[i].setText("Pattern "+Integer.toString(i+1)+" value");
		textFieldArray[i]=new JTextField("70",0);
		textFieldArray[i].setEnabled(false);
		textFieldArray[i].setPreferredSize(new Dimension(50, 25));
		textFieldArray[i].setHorizontalAlignment(SwingConstants.CENTER);
		final JTextField tempText = textFieldArray[i];
		textFieldArray[i].setDocument(new PlainDocument(){
			private static final long serialVersionUID = 200601l;
			public void insertString(int offs, String str, AttributeSet a)throws BadLocationException{
			    replace(offs,0,str,a);
			}
			public void remove(int offs, int len)throws BadLocationException{
			    replace(offs,len,"",null);
			}
			public void replace(int offs,int len,String text,AttributeSet a)throws BadLocationException{
			    String s = tempText.getText();
			    String newText = s.substring(0,offs)+text+s.substring(offs+len);
			    if(newText.equals("")){
				super.remove(offs,len);
				super.insertString(offs,text,a);
			    }else {
				try{
				    float d = Float.parseFloat(newText);
				    super.remove(offs,len);
				    super.insertString(offs,text,a);
				}catch(Exception e){}				
			    }
			}
		    });
		comboBoxArray[i]=new JComboBox(new DefaultComboBoxModel(bvVector));

		JPanel temp = new JPanel();
		temp.setSize(300,30);
		temp.add(radButtonArray[i]);
		temp.add(comboBoxArray[i]);
		temp.add(textFieldArray[i]);
		temp.add(new JLabel("%"));
		jPanel4.add(temp);
	}
	BVlabel=new String[Nsol];
	isBV=false;
	is3d=false;
	    
	graphChooser.setVisible(true);
	this.getContentPane().add(graphChooser, BorderLayout.NORTH);
	ic.setDisplayDataNames(false);
	ic.clearGraphData();
	ic.resetGraph();
	ic.repaint();
    }

    /*
    * computes the chi squared value from the mock and data
    */
    private float computeChiSquared(){
	float chiSquared=0;
	for(int i=0;i<Na;i++){
	    for(int j=0;j<Nf;j++){
	        chiSquared+=(((float[][])dataArray.get(2))[i][j]*((float[][])dataArray.get(2))[i][j])/(((float[][])dataArray.get(5))[i][j]*((float[][])dataArray.get(5))[i][j]);
	    }
	}
	return chiSquared;
    }

    /*
    * prints the graph
    */
    private void print_actionPerformed(ActionEvent e){
	PrinterJob printJob = PrinterJob.getPrinterJob();
	PageFormat pf=new PageFormat();
	pf.setOrientation(PageFormat.LANDSCAPE);
	printJob.setPrintable(ic,pf);
	if(printJob.printDialog()){
		try{
			printJob.print();
		}catch(PrinterException pe){
			JOptionPane.showMessageDialog(this,pe,"WARNING",JOptionPane.WARNING_MESSAGE);
		}
	}
	ic.draw();
    }

    private void plot_actionPerformed(ActionEvent e){
	float[][] dat = new float[Na][3];
	isBV=false;
	is3d=false;
	print.setVisible(false);
	try{
	    g3d.removePoints();
	    g3d.reset();
	    g3d.setLabels(indices[0].getText(),indices[1].getText(),indices[2].getText());
	    for(int i = 0;i<dat.length;i++){
		for(int j=0;j<dat[i].length;j++){
		    dat[i][j]=((float[][])dataArray.get(4))[i][Integer.parseInt(indices[j].getText().equals("")?"0":indices[j].getText())];
		}
		//dat[i][0]=((float[][])dataArray.get(4))[i][Integer.parseInt(indices[0].getText())];
		//dat[i][1]=((float[][])dataArray.get(4))[i][Integer.parseInt(indices[1].getText())];
		//dat[i][2]=((float[][])dataArray.get(4))[i][Integer.parseInt(indices[2].getText())];
	    }
	    g3d.addPoints(dat);
	    right.setVisible(false);
	    left.setVisible(false);
	    label.setVisible(false);
	    cl.show(lp,"g3d");
	}catch(IndexOutOfBoundsException exception){
	    JOptionPane.showMessageDialog(this,exception,"WARNING",JOptionPane.WARNING_MESSAGE);
	}
	validate();
	frame3D.dispose();
    }

    /*
    * creates 3D graph
    */
    private void graph3d_actionPerformed(ActionEvent e){
	frame3D.setVisible(true);
    }

    /*
    * saves the selected data to a tab-delimited file
    */
    private void export_actionPerformed(ActionEvent e){
	saveOptions[8].setVisible(isBV);
	saveOptions[9].setVisible(is3d);
	for(int i=0;i<saveOptions.length;i++){
	    saveOptions[i].setSelected(false);
	}

	if(jfcout.showSaveDialog(this) == JFileChooser.APPROVE_OPTION){
	    try{
		BufferedWriter bw = new BufferedWriter(new FileWriter(jfcout.getSelectedFile()));

		bw.write(filename);
		bw.newLine();
		bw.write(new Integer(chiSq).toString());
		bw.newLine();
		bw.write(date);
		for(int i=0;i<saveOptions.length;i++){
		    if(saveOptions[i].isSelected()){

			bw.newLine();
			bw.newLine();
			bw.write(saveOptions[i].getText());
			bw.newLine();
			if(i<8){
			    for(int j=0;j<((float[][])dataArray.get(i)).length;j++){
				if(geneNames!=null&&geneNames.length==Na){
				    if(i!=3 &&i!=6){
				        bw.write(geneNames[j]+'\t');
				    }else{
					bw.write('\t');
				    }
				}else{
				    if(i!=3 &&i!=6){
				        bw.write(Integer.toString(j)+'\t');
				    }else{
					bw.write('\t');
				    }
				}
				for(int k=0;k<((float[][])dataArray.get(i))[0].length;k++){
				    bw.write(((float[][])dataArray.get(i))[j][k]+"\t");
				}
				bw.newLine();
			    }
			}else if(i==9){
			    for(int j=0;j<index.length;j++){
				if(geneNames!=null&&geneNames.length==Na){
				    bw.write(geneNames[index[j]]+'\t');
				}else{
				    bw.write(Integer.toString(index[j])+'\t');
				}
				for(int k=0;k<((float[][])dataArray.get(1))[0].length;k++){
				    bw.write(((float[][])dataArray.get(1))[index[j]][k]+"\t");
				}
				bw.newLine();
			    }
			}else{
			    int[] temp;
			    for(int j=0;j<radButtonArray.length;j++){
				if(BVlabel[j]!=NOT_SELECTED){
				    bw.write(BVlabel[j]);
				    bw.newLine();
				}
			    }
			    temp=dataLabels;
			    for(int j=0;j<temp.length;j++){
				if(geneNames!=null&&geneNames.length==Na){
				    bw.write(geneNames[temp[j]]+'\t');
				}else{
				    bw.write(Integer.toString(temp[j])+'\t');
				}
				for(int k=0;k<Nf;k++){
				    bw.write(((float[][])dataArray.get(1))[temp[j]][k]+"\t");
				}
				bw.newLine();
			    }			    
			}
		    }
		}
		bw.close();
	    }catch(IOException exception){
		JOptionPane.showMessageDialog(this,exception,"WARNING",JOptionPane.WARNING_MESSAGE);
	    }
	}
    }

    /*
    * gets the data to be graphed and then calls makeGraph
    */
    private void graphChoice_actionPerformed(ActionEvent e) {
	ic.setDisplayDataNames(false);

	int numGraph=0;
	for(int i=0;i<buttonArray.length;i++){
		if(buttonArray[i].isSelected()){
			numGraph++;
		}
	}
	int[] dataLabel=new int[numGraph];
	
	int labelNo=0;
	for(int i=0;i<buttonArray.length;i++){
		if(buttonArray[i].isSelected()){
			dataLabel[labelNo]=i;
			labelNo++;
		}
	}

	if(numGraph!=0){
		isBV=false;
		is3d=false;
		String[] temp=new String[dataLabel.length];
		for(int i=0;i<dataLabel.length;i++){
			temp[i]=graphNames[dataLabel[i]];
		}
		dataLabels=dataLabel;
		ic.setDataLabels(temp);
		isCol=!(colRow.getSelectedIndex()==0);
		gene=0;
		makeGraph();
	}
    }

    /*
    * graphs the selected data on ic
    */
    protected void makeGraph(){
	cl.show(lp,"ic");
	print.setVisible(true);
	ic.setDisplayZeroFlag(false);
	ic.clearGraphData();
	String[] y;
	if(isBV){
		if(xLabels!=null&&xLabels.length==Nf){
			y=xLabels;
		}else{
			y = new String[((float[][])dataArray.get(0))[dataLabels[0]].length];
			for(int i=0; i<y.length; i++){
				y[i]=String.valueOf(i+1);
			}
		}
		ic.removeDataLabels();
		for(int i=0;i<dataLabels.length;i++){
			ic.addData(((float[][])dataArray.get(1))[dataLabels[i]],y);
		}
	}else{
		if(xLabels!=null&&xLabels.length==Nf&&!isCol&&dataLabels[0]!=4){
			y=xLabels;
		}else{
			y = new String[colToRow((float[][])dataArray.get(dataLabels[0]),isCol)[0].length];
			for(int i=0; i<y.length; i++){
				y[i]=String.valueOf(i+1);
			}
		}
		String[] tempNameArray=new String[dataLabels.length];
		for(int i=0; i<dataLabels.length;i++){
			ic.addData(colToRow((float[][])dataArray.get(dataLabels[i]),isCol)[gene],y);
			if(geneNames!=null&&geneNames.length==Na){
				tempNameArray[i]=geneNames[Math.min(gene,geneNames.length-1)];
			}
			if(dataLabels[i]==2){
		        	ic.addErrorBars(colToRow((float[][])dataArray.get(5),isCol)[gene],i);
				ic.setDisplayZeroFlag(true);
			}
		}
		if(dataLabels[0]!=3&&geneNames!=null&&geneNames.length==Na&&!isCol){
			ic.setDataNames(tempNameArray);
			ic.setDisplayDataNames(true);
		}
		right.setEnabled(gene !=(colToRow((float[][])dataArray.get(dataLabels[0]),isCol).length-1));
		left.setEnabled(gene!=0);
	        label.setText(Integer.toString(gene+1));
	}
	left.setVisible(!isBV);
	right.setVisible(!isBV);
	label.setVisible(!isBV);
	ic.draw();
	ic.repaint();
	validate();
    }

    /*
    * reverses the columns and rows if requested
    */
    private float[][] colToRow(float[][] initialArray, boolean isSwitched){
	if(isSwitched){
	    float[][] finalArray=new float[initialArray[0].length][initialArray.length];
	    for(int i=0;i<initialArray.length;i++){
		for(int j=0;j<initialArray[0].length;j++){
		    finalArray[j][i]=initialArray[i][j];
		}
	    }
	    return finalArray;
	}
	return initialArray;
    }

    /*
    * deselects other checkboxes if necessary
    * choosing mock, data, or residuals deselects pattern and distribution
    * choosing pattern or distribution deselects mock, data, and residuals
    */
    private void Checkbox_actionPerformed(ActionEvent e) {
	if(((JCheckBox)e.getSource()).equals(buttonArray[0])||((JCheckBox)e.getSource()).equals(buttonArray[1])||((JCheckBox)e.getSource()).equals(buttonArray[2])){
	    buttonArray[3].setSelected(false);
	    buttonArray[4].setSelected(false);
	}else{
	    buttonArray[0].setSelected(false);
	    buttonArray[1].setSelected(false);
	    buttonArray[2].setSelected(false);
	    if(((JCheckBox)e.getSource()).equals(buttonArray[3])){
		buttonArray[4].setSelected(false);
	    }else{
		buttonArray[3].setSelected(false);
	    }
	}
    }

    /*
    * increments or decrements gene depending on isRight and calls makeGraph
    */
    public void rl_actionPerformed(boolean isRight){
	    if(isRight){
		gene++;
	    }else{
		gene--;
	    }
	    makeGraph();
    }

    /*
    * changes gene to either the number type into the field or the number 
    * associated with the gene name.  If the input is invalid, gene doesn't 
    * change.
    * makeGraph is then called.
    */
    private void label_actionPerformed(ActionEvent e){
	try{
	    if(Integer.valueOf(label.getText()).intValue()-1<=(colToRow(((float[][])dataArray.get(dataLabels[0])),isCol).length-1)&&Integer.valueOf(label.getText()).intValue()-1>=0){
	        gene=Integer.valueOf(label.getText()).intValue()-1;
	    }
	    makeGraph();
	}catch(NumberFormatException exception){
	    boolean isGeneName=false;
	    if(geneNames!=null&&geneNames.length==Na&&!isCol&&dataLabels[0]!=3&&!label.getText().equals("")){
                for(int i=1;i<geneNames.length+1;i++){
                    if(geneNames[(i+gene)%geneNames.length].toLowerCase().indexOf(label.getText().toLowerCase())==0){
			gene=(i+gene)%geneNames.length;
			makeGraph();
			isGeneName=true;
                        break;
                    }
                }
	    }  
	    if(!isGeneName){
		label.setText(Integer.toString(gene+1));
	    }
	}
    }

    /*
    * shows the basis vector frame
    */
    private void bVect_actionPerformed(ActionEvent e) {
	basisVect.setVisible(true);
    }

    /*
    * action method for each of the radio buttons.
    * simply activates the corresponding text pane
    */
    private void tempButton_actionPerformed(ActionEvent e){
	int i=0;
	for(int j=0;j<Nsol;j++){
	    if(e.getSource().equals(radButtonArray[j])){
		i=j;
	    }
	}
	textFieldArray[i].setEnabled(((JCheckBox)e.getSource()).isSelected());
    }

    /*
    * finds the desired data and calls makeGraph to display it
    */
    private void bvGraph_actionPerformed(ActionEvent e) {
	int numSets=Na;
	float bvData;
	boolean[] boolArray=new boolean[Na];
	float[][] holder = scaleData((float[][])dataArray.get(4));
	for(int i = 0;i<Na;i++){boolArray[i]=true;}
	for(int i=0;i<Nsol;i++){
	    if(radButtonArray[i].isSelected()){
		try{
		    bvData=(Float.valueOf(textFieldArray[i].getText())).floatValue();
		    for(int j=0;j<Na;j++){
			if(boolArray[j]){
			    if(comboBoxArray[i].getSelectedIndex()==0){
		        	boolArray[j]=holder[j][i]>=(bvData/100.0f);
			    }else{
			        boolArray[j]=holder[j][i]<=(bvData/100.0f);
			    }
		    	if(!boolArray[j]){
				numSets--;
			    }
			}
		    }
		}catch(NumberFormatException exception){
			textFieldArray[i].setText("0");
		}
	    }
	}		
	if(numSets==0){
	    JOptionPane.showMessageDialog(this,"No genes meet specifications","WARNING",JOptionPane.WARNING_MESSAGE);
	}else{
	    int counter=0;
	    int[] intArray=new int[numSets];
	    String[] geneLabels=new String[numSets];
	    for(int i=0;i<Na;i++){
	        if(boolArray[i]){
		    intArray[counter]=i;
		    if(geneNames!=null&&geneNames.length==Na){
		        geneLabels[counter]=geneNames[i];
		    }
		    counter++;
	        }
	    }  
	    isBV=true;
	    is3d=false;
	    dataLabels=intArray;
	    for(int j=0;j<radButtonArray.length;j++){
		BVlabel[j]=(radButtonArray[j].isSelected())?"Pattern "+(j+1)+((comboBoxArray[j].getSelectedIndex()==0)?" over ":" under ")+textFieldArray[j].getText()+"%":NOT_SELECTED;
	    }
	    if(geneNames!=null&&geneNames.length==Na){
		ic.setDataNames(geneLabels);
		ic.setDisplayDataNames(true);
	    }
	    makeGraph();
	}
	basisVect.dispose();
    }   

    /*
    * sets the distribtion values to percentages for Basis Vect
    */
    private float[][] scaleData(float[][] rData){
	float[][] temp=new float[rData.length][rData[0].length];
	for(int i=0;i<rData.length;i++){
	    float total=0;
	    for(int j=0;j<rData[0].length;j++){
		total+=rData[i][j];
	    }
	    for(int j=0;j<rData[0].length;j++){
		temp[i][j]=rData[i][j]/total;
	    }
	}
	return temp;
    }

    public static void main(String argv[]) {
	PattFrame pattview = new PattFrame();
	pattview.setVisible(true);
    }    
}
