// Class: BitmapDraw
//
// Purpose: create an Image object from a bitmap
// 
// History: 12/1/99 jac written to seperate image construction 
//                   from image display for bitmap objects
//
// COPYRIGHT, 2000, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package viewer.csi.gui;


import java.awt.Image;

import java.awt.Graphics;

import java.awt.Dimension;

import java.awt.Color;

import java.awt.Insets;

import java.awt.Point;

import java.awt.image.MemoryImageSource;


import java.lang.String;

import java.lang.Float;

import java.lang.Math;


import java.util.*;

import javax.swing.JPanel;


public class BitmapDraw implements viewer.csi.gui.csiConstants {


    public static int version = 200001;

    public static String copyright = "Copyright 2000, Fox Chase Cancer Center";


    // Create a java Image file from BitmapObject's real or imag part
    public static Image drawBitmapData(JPanel canPanel,Image im, float[][] data) {

	
	float min = Float.MAX_VALUE, max = -Float.MAX_VALUE;
	int width, height, bitmapWidth, bitmapHeight, col;
	int[] grayScale = new int[GRAY_SHADES];

	// get size of canvas panel on screen for drawing
	Dimension dim = canPanel.getSize();

	Insets ins = canPanel.getInsets();

	if(dim.width <= 0)
	    dim.width = 10;
	if(dim.height <= 0)
	    dim.height = 10;
	width = dim.width - ins.left - ins.right;

	height = dim.height - ins.top - ins.bottom;


        // use 256 shades of gray for display
	int i,j;

	float deltaBright = 255f / (GRAY_SHADES - 1);
	for(i=0; i<GRAY_SHADES; i++)  {

	    j = (int)(deltaBright * i);

	    grayScale[i] = (new Color(j, j, j).getRGB());

	}

	

	// draw the data

	bitmapWidth = data[0].length;

	bitmapHeight = data.length;

	

	int[] bitmapArray = new int[bitmapWidth * bitmapHeight];

	

	for (i=0; i<bitmapHeight; i++) {  // find max and min
	    for (j=0; j<bitmapWidth; j++) {

		if (max < data[i][j])

		    max = data[i][j];

		if (min > data[i][j])

		    min = data[i][j];

	    }

	}

	
	for (i=0; i<bitmapHeight; i++) { // scale data to 256 shades
	    for (j=0; j<bitmapWidth; j++) {
		col = (int)((GRAY_SHADES * (data[i][j] - min))/(max-min));
		if (col >= GRAY_SHADES)
		    col = GRAY_SHADES - 1;
		if (col < 0)
		    col = 0;
		// these appear to be the actual colorings
		bitmapArray[(i * bitmapWidth) + j] = grayScale[col];
	    }
	}

	Image bitmapImage = canPanel.createImage(new MemoryImageSource(bitmapWidth,bitmapHeight,bitmapArray,0,bitmapWidth));

	if (width < height)
	    height = width;
	else
	width = height;
	return bitmapImage.getScaledInstance(width,height,Image.SCALE_FAST);

    }

}

    

