// Class: DrawingMethods
//
// Purpose: static methods to process drawing events like grids, 
//           coordinate display, and coordinate lines
//
// History: 1-6-00, jac
//
// COPYRIGHT, 2000, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program;  if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package viewer.csi.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Insets;
import java.awt.FontMetrics;

public class DrawingMethods {

    public static void fillIn( Graphics g, Point a, Point b ) {
	int firstX = a.x;
	int secondX = (b.x - a.x);
	int firstY = a.y;
	int secondY = (b.y - a.y);

	if (b.x < a.x) {
	    firstX = b.x;
	    secondX = (a.x - b.x);
	}

	if (b.y < a.y) {
	    firstY = b.y;
	    secondY = (a.y - b.y);
	}

	g.setColor(new Color(192,192,192,75));
	g.fillRect(firstX,firstY,secondX,secondY);
    }
       
    public static void drawGrid( Graphics g, int canWidth, int canHeight, int xLines, int yLines, int xSel, int ySel, float xoff, float yoff ) {
	int xSkip = canWidth/(xLines + 1);
	int ySkip = canHeight/(yLines + 1);

	int xOffset = (int)((float)xSkip * xoff);
	int yOffset = (int)((float)ySkip * yoff);

	Color myGray = new Color(192,192,192,75);

	g.setColor(myGray);

	for (int i=0; i<xLines; i++) {
	    g.translate(xSkip,0);
	    if (i == xSel) {
		g.setColor(Color.cyan);
	    }
	    g.drawLine(0 + xOffset,0,0 + xOffset,canHeight);
	    g.setColor(myGray);
	}

	g.translate(-(xSkip*xLines),0);

	for (int i=0; i<yLines; i++) {
	    g.translate(0,ySkip);
	    if (i == ySel) {
		g.setColor(Color.cyan);
	    }
	    g.drawLine(0,0 + yOffset,canWidth,0 + yOffset);
	    g.setColor(myGray);
	}

	g.translate(0,-(ySkip*yLines));
    }

    public static void drawCoordLines( Graphics g, Insets ins, int canWidth, int canHeight, Point a, Color lineColor) {
	int top = ins.top;
	int left = ins.left;
	int mouseTop = a.y + top;
	int mouseLeft = a.x + left;
	int heightTop = canHeight - 1 + top;
	int widthLeft = canWidth - 1 + left;

	g.setColor(lineColor);
	g.drawLine(left,mouseTop,widthLeft,mouseTop);
	g.drawLine(mouseLeft,top,mouseLeft,heightTop);
    }

    public static void displayCoordinates( Graphics g, Insets ins, FontMetrics fmt, int canWidth, String[] xLabels, float[] data, int vPoint, Point a, Color[] suite) {
	displayDataNames(g,ins,fmt,canWidth,xLabels,xLabels[vPoint] + ", " + String.valueOf(data[vPoint]),vPoint,a,suite);
    }
    public static void displayDataNames( Graphics g, Insets ins, FontMetrics fmt, int canWidth, String[] xLabels, String dataName, int vPoint, Point a, Color[] suite) {
	int left = ins.left;
	int top = ins.top;
	int mouseLeft = left + a.x;
	int mouseTop = top + a.y;
	int textLeft = mouseLeft + 10;
	int textTop = mouseTop - 10;
	int pointLineX = textLeft - 2;
	int pointLineY = textTop - 1;
	int pointLineX2 = textLeft + 5;
	int pointLineY2 = textTop - 5;
	String s = dataName;
	int stringWidth = fmt.stringWidth(s);
	int stringHeight = fmt.getHeight();

	if (textLeft + stringWidth > canWidth + left) {
	    textLeft = mouseLeft - 10 - stringWidth + 1;
	    pointLineX = mouseLeft - 10 + 2;
	    pointLineX2 = mouseLeft - 5;
	}
	if (textTop - stringHeight < top) {
	    textTop = mouseTop + 10 + stringHeight - 4;
	    pointLineY = mouseTop + 10;
	    pointLineY2 = mouseTop + 5;
	}

	g.setColor(suite[5]);
	g.drawLine(pointLineX,pointLineY,pointLineX2,pointLineY2);
	g.fillRect(textLeft - 2,textTop - stringHeight + 3, stringWidth + 4, stringHeight - 1);
	g.setColor(suite[6]);
	g.drawString(s,textLeft,textTop);
    }
    public static int[] getRectangle(Point p1,Point p2,int dataLength) {
	if (p1 != null && p2 != null) {
	    int[] res = new int[6];
	    if (p1.x < p2.x) {
		res[0] = p1.x;
		res[2] = p2.x;
	    } else {
		res[0] = p2.x;
		res[2] = p1.x;
	    }
	    
	    if (p1.y < p2.y) {
		res[1] = p1.y;
		res[3] = p2.y;
	    } else {
		res[1] = p2.y;
		res[3] = p1.y;
	    }
		
	    if (res[0] < 0) 
		    res[0] = 0;

	    if (res[2] >= dataLength)
		res[2] = dataLength - 1;
	    
	    res[4] = res[2] - res[0] + 1;
	    res[5] = res[3] - res[1] + 1;
	    return res;
	} else {
	    return null;
	}
    } 
}
    
    
