// Class: ImageCanvas
//
// Purpose: creates two-dimensional graphs which are resizable and printable
//
// History: v1.0 8/99 jac rewritten from old version
//          v2.0 5/02 WFS
//
// COPYWRIGHT, 2002, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the license, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have recieved a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package viewer.csi.gui;

import java.awt.Image;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Insets;
import java.awt.Point;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.GraphicsConfiguration;
import java.awt.image.MemoryImageSource;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.awt.print.*;

import javax.swing.JPanel;
import javax.swing.JDialog;
import javax.swing.border.Border;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Box.Filler;
import javax.swing.JLabel;
import javax.swing.JRootPane;
import javax.swing.RepaintManager;

import java.util.*;

import viewer.csi.gui.VectorDraw;
import viewer.csi.gui.DrawingMethods;
import viewer.csi.gui.event.*;

public class ImageCanvas extends javax.swing.JPanel implements viewer.csi.gui.csiConstants, Printable {
    public static int version = 19990002;
    public static String copyright = "Copyright 1999, Fox Chase Cancer Center";

    protected JPanel scalesPanel;
    // labels which work with setLabels to add strings around the borders and/or in the corners
    protected JLabel label1,label2,label3,label4,label5,label6,label7,label8,label9,label10,label11,label12;
    protected JLabel bottomLeftCorner,bottomRightCorner,upperLeftCorner,upperRightCorner;

    // glues keep the JLabels in place during resize
    protected Component vGlue2,vGlue4,vGlue6,vGlue8,vGlue10,vGlue12,vGlue14,vGlue16;
    protected Component hGlue10,hGlue12,hGlue14,hGlue16,hGlue18,hGlue20,hGlue22,hGlue24;
    protected Component leftStrut,leftStrut1,leftStrut2,rightStrut,rightStrut1,rightStrut2;

    // Boxes contain JLabels
    protected Box leftSideLabels,rightSideLabels,upperSideLabels,bottomSideLabels,upperRightPanel,upperLeftPanel,bottomLeftPanel,bottomRightPanel;

    // display variables
    protected canvasPanel display = new canvasPanel();
    protected Image canvasImage = null;
    protected Image canvasImage2 = null;

    protected int dataType = TYPE_NODATA;

    protected boolean displayZeroFlag = false, drawCoordinateLines = false, displayCoordinates = false, displayDataLabels=false, displayErrorBars[], displayDataNames=false;
    protected boolean displayScale = true, grid = false, fillIn = false;

    protected Point mousePoint = new Point(0,0), mouseStartPoint = new Point(0,0), fillA = new Point(0,0), fillB = new Point(0,0);
    protected Point actualMousePoint = new Point(0,0);
    protected int canvasWidth=0,canvasHeight=0;
    protected boolean focusFlag = false;

    protected String[] xScaleLabels, yScaleLabels, cornerLabels, borderLabels, dataLabels;

    protected float[] noise;
    protected String[] dataNames;

    // listener vectors
    protected Vector<PointChangedListener> pointListeners;
    protected Vector<ZoomListener> zoomListeners;
    protected Vector<ActiveICChangedListener> aICCListeners;
    protected Vector<MarkListener> markListeners;
    
    // vector variables
    protected int startPoint = 0, numPoints, endPoint;
    protected float vRangeMax, vRangeMin;
    protected int vectorPoint;
    protected float[] vData,curData;
    protected String[] scaleX,scaleY;
    protected ArrayList<float[]> multData=new ArrayList<float[]>(2);

    // bitmap variables
    protected Point bitmapCursorPosition = new Point(0,0), bitmapStartCursorPosition = new Point(0,0);
    protected int bitmapWidth,bitmapHeight;
    protected float[][] bData;

    // grid variables
    protected int numbXLines,numbYLines,xSel,ySel;
    protected float xOffset = 0,yOffset = 0;

    // horizontal line variables
    protected float[]     hTLines = new float[10];
    protected boolean[]   hTLFlag = new boolean[10];
    protected Color[]     hTLColor = new Color[10];

    protected Color[] colorSuite = {Color.black,Color.cyan,Color.red,Color.yellow,Color.green,Color.white,Color.black,Color.blue,Color.black};
    // arranged as background,vector line,,,vector scale,coordinate background,coordinate foreground,,unselected border

    public ImageCanvas()  {
	super();
	setOpaque(false);

	setDoubleBuffered( false );
	setLayout( new java.awt.BorderLayout() );
	scalesPanel = new JPanel();
	scalesPanel.setOpaque(false);
	scalesPanel.setDoubleBuffered( false );
	scalesPanel.setLayout( new java.awt.BorderLayout() );
	leftSideLabels = new Box( BoxLayout.Y_AXIS );
	leftStrut = Box.createHorizontalStrut( 0 );
	((Filler)leftStrut).changeShape(new Dimension( 0, 0 ), new Dimension( 0, 0 ), new Dimension( 0, 0 ) );
	vGlue2 = Box.createVerticalGlue();
	label12 = new JLabel();
	label12.setText( "" );
	vGlue4 = Box.createVerticalGlue();
	label11 = new JLabel();
	label11.setText( "" );
	vGlue8 = Box.createVerticalGlue();
	label10 = new JLabel();
	label10.setText( "" );
	vGlue6 = Box.createVerticalGlue();
	leftSideLabels.add(leftStrut, null, -1);
	leftSideLabels.add(vGlue2, null, -1);
	leftSideLabels.add(label12, null, -1);
	leftSideLabels.add(vGlue4, null, -1);
	leftSideLabels.add(label11, null, -1);
	leftSideLabels.add(vGlue8, null, -1);
	leftSideLabels.add(label10, null, -1);
	leftSideLabels.add(vGlue6, null, -1);
	rightSideLabels = new Box( BoxLayout.Y_AXIS );
	rightStrut = Box.createHorizontalStrut( 0 );
	((Filler)rightStrut).changeShape(new Dimension( 0, 0 ), new Dimension( 0, 0 ), new Dimension( 0, 0 ) );
	vGlue10 = Box.createVerticalGlue();
	label4 = new JLabel();
	label4.setText( "" );
	vGlue12 = Box.createVerticalGlue();
	label5 = new JLabel();
	label5.setText( "" );
	vGlue14 = Box.createVerticalGlue();
	label6 = new JLabel();
	label6.setText( "" );
	vGlue16 = Box.createVerticalGlue();
	rightSideLabels.add(rightStrut, null, -1);
	rightSideLabels.add(vGlue10, null, -1);
	rightSideLabels.add(label4, null, -1);
	rightSideLabels.add(vGlue12, null, -1);
	rightSideLabels.add(label5, null, -1);
	rightSideLabels.add(vGlue14, null, -1);
	rightSideLabels.add(label6, null, -1);
	rightSideLabels.add(vGlue16, null, -1);
	upperSideLabels = new Box( BoxLayout.X_AXIS );
	upperLeftPanel = new Box( BoxLayout.Y_AXIS );
	leftStrut1 = Box.createHorizontalStrut( 5 );
	((Filler)leftStrut1).changeShape(new Dimension( 5, 0 ), new Dimension( 5, 0 ), new Dimension( 5, 0 ) );
	upperLeftCorner = new JLabel();
	upperLeftCorner.setText( "" );
	upperLeftPanel.add(leftStrut1, null, -1);
	upperLeftPanel.add(upperLeftCorner, null, -1);
	hGlue10 = Box.createHorizontalGlue();
	label1 = new JLabel();
	label1.setText( "" );
	hGlue12 = Box.createHorizontalGlue();
	label2 = new JLabel();
	label2.setText( "" );
	hGlue14 = Box.createHorizontalGlue();
	label3 = new JLabel();
	label3.setText( "" );
	hGlue16 = Box.createHorizontalGlue();
	upperRightPanel = new Box( BoxLayout.Y_AXIS );
	rightStrut1 = Box.createHorizontalStrut( 5 );
	((Filler)rightStrut1).changeShape(new Dimension( 5, 0 ), new Dimension( 5, 0 ), new Dimension( 5, 0 ) );
	upperRightCorner = new JLabel();
	upperRightCorner.setText( "" );
	upperRightPanel.add(rightStrut1, null, -1);
	upperRightPanel.add(upperRightCorner, null, -1);
	upperSideLabels.add(upperLeftPanel, null, -1);
	upperSideLabels.add(hGlue10, null, -1);
	upperSideLabels.add(label1, null, -1);
	upperSideLabels.add(hGlue12, null, -1);
	upperSideLabels.add(label2, null, -1);
	upperSideLabels.add(hGlue14, null, -1);
	upperSideLabels.add(label3, null, -1);
	upperSideLabels.add(hGlue16, null, -1);
	upperSideLabels.add(upperRightPanel, null, -1);
	bottomSideLabels = new Box( BoxLayout.X_AXIS );
	bottomLeftPanel = new Box( BoxLayout.Y_AXIS );
	leftStrut2 = Box.createHorizontalStrut( 5 );
	((Filler)leftStrut2).changeShape(new Dimension( 5, 0 ), new Dimension( 5, 0 ), new Dimension( 5, 0 ) );
	bottomLeftCorner = new JLabel();
	bottomLeftCorner.setText( "" );
	bottomLeftPanel.add(leftStrut2, null, -1);
	bottomLeftPanel.add(bottomLeftCorner, null, -1);
	hGlue24 = Box.createHorizontalGlue();
	label9 = new JLabel();
	label9.setText( "" );
	hGlue18 = Box.createHorizontalGlue();
	label8 = new JLabel();
	label8.setText( "" );
	hGlue22 = Box.createHorizontalGlue();
	label7 = new JLabel();
	label7.setText( "" );
	hGlue20 = Box.createHorizontalGlue();
	bottomRightPanel = new Box( BoxLayout.Y_AXIS );
	rightStrut2 = Box.createHorizontalStrut( 5 );
	((Filler)rightStrut2).changeShape(new Dimension( 5, 0 ), new Dimension( 5, 0 ), new Dimension( 5, 0 ) );
	bottomRightCorner = new JLabel();
	bottomRightCorner.setText( "" );
	bottomRightPanel.add(rightStrut2, null, -1);
	bottomRightPanel.add(bottomRightCorner, null, -1);
	bottomSideLabels.add(bottomLeftPanel, null, -1);
	bottomSideLabels.add(hGlue24, null, -1);
	bottomSideLabels.add(label9, null, -1);
	bottomSideLabels.add(hGlue18, null, -1);
	bottomSideLabels.add(label8, null, -1);
	bottomSideLabels.add(hGlue22, null, -1);
	bottomSideLabels.add(label7, null, -1);
	bottomSideLabels.add(hGlue20, null, -1);
	bottomSideLabels.add(bottomRightPanel, null, -1);

	String strConstraint;
	strConstraint = "Center";
	this.add(scalesPanel, strConstraint, -1);
	strConstraint = "West";
	this.add(leftSideLabels, strConstraint, -1);
	strConstraint = "East";
	this.add(rightSideLabels, strConstraint, -1);
	strConstraint = "North";
	this.add(upperSideLabels, strConstraint, -1);
	strConstraint = "South";
	this.add(bottomSideLabels, strConstraint, -1);
	strConstraint = "Center";
	scalesPanel.add(display, strConstraint, -1);
	
	// set canvas size variables
	Dimension dim = display.getSize();
	Insets ins = display.getInsets();
	if(dim.width <= 0)
	    dim.width = 10;
	if(dim.height <= 0)
	    dim.height = 10;
	canvasWidth = dim.width - ins.left - ins.right;
	canvasHeight = dim.height - ins.top - ins.bottom;
    }

    public void setData(float[] dat, String[] scl) {
	vData = dat;
	scaleX = scl;
	bData = null;
	scaleY = null;
	curData = vData;
        multData.add(vData);
	numPoints = curData.length;
	startPoint = 0;
	endPoint = numPoints - 1;
	dataType = TYPE_VECTOR;
	displayErrorBars=new boolean[multData.size()];
	for(int i=0;i<displayErrorBars.length;i++){
	    displayErrorBars[i]=false;
	}
	createScaleLabels();
	calculateVerticalRange();
	draw();
    }

    public void addData(float[] dat, String[] scl) {
        if(multData.size()>0 && dat.length != vData.length){
            multData.clear();
        }
        multData.add(dat);
	vData = dat;
	scaleX = scl;
	bData = null;
	scaleY = null;
	curData = vData;
	numPoints = curData.length;
	startPoint = 0;
	endPoint = numPoints - 1;
	dataType = TYPE_VECTOR;
	displayErrorBars=new boolean[multData.size()];
	for(int i=0;i<displayErrorBars.length;i++){
	    displayErrorBars[i]=false;
	}
	createScaleLabels();
	calculateVerticalRange();
	mousePoint = new Point(0,0);
	vectorPoint = 0;
	//draw();
    }


    public void addData(ArrayList<float[]> dataList, String[] scl) {
        multData=dataList;
	vData = (float[])multData.get(multData.size()-1);
	scaleX = scl;
	bData = null;
	scaleY = null;
	curData = vData;
	numPoints = curData.length;
	startPoint = 0;
	endPoint = numPoints - 1;
	displayErrorBars=new boolean[multData.size()];
	for(int i=0;i<displayErrorBars.length;i++){
	    displayErrorBars[i]=false;
	}
	dataType = TYPE_VECTOR;
	createScaleLabels();
	calculateVerticalRange();
	//draw();
    }

    public void clearGraphData(){
        multData.clear();
	dataType=TYPE_NODATA;
    }

    public void resetGraph(){
	canvasImage=null;
    }

    public void setData(float[] dat) {
	vData = dat;
	curData = vData;
	numPoints = curData.length;
	startPoint = 0;
	endPoint = numPoints - 1;
	dataType = TYPE_VECTOR;
	draw();
    }

    public void setData(float[][] dat) {
	bData = dat;
	bitmapWidth = bData[0].length;
	bitmapHeight = bData.length;
	dataType = TYPE_BITMAP;
    }

    public void setDataLabels(String[] dataSetLabels){
	dataLabels=dataSetLabels;
	setDisplayDataLabels(true);
    }

    public void removeDataLabels(){
	dataLabels=null;
	setDisplayDataLabels(false);
    }

    public void addErrorBars(float[] noiseF,int i){
	noise=noiseF;
	setDisplayErrorBars(true,i);
    }

    public void removeErrorBars(){
	noise=null;
	for(int i=0;i<displayErrorBars.length;i++){
	    setDisplayErrorBars(false,i);
	}
    }

    public void setDataNames(String[] dataName){
	dataNames=dataName;
    }

    public float[] getData() {
	return curData;
    }

    public void setScale(String[] scl) {
	scaleX = scl;
	scaleY = null;
	createScaleLabels();
	draw();
    }

    public void setScale(String[] sclX, String[] sclY) {
	scaleX = sclX;
	scaleY = sclY;
    }

    public void dispScale(boolean b) {
	displayScale = b;
    }

    protected void setLabels(String[] newCorner, String[] newBorder) {
	cornerLabels = newCorner;
	borderLabels = newBorder;

	if (cornerLabels != null && cornerLabels.length == 4) {
	    upperLeftCorner.setText(cornerLabels[0]);
	    upperRightCorner.setText(cornerLabels[1]);
	    bottomRightCorner.setText(cornerLabels[2]);
	    bottomLeftCorner.setText(cornerLabels[3]);
	} else {
	    upperLeftCorner.setText(null);
	    upperRightCorner.setText(null);
	    bottomRightCorner.setText(null);
	    bottomLeftCorner.setText(null);
	}

	upperLeftPanel.validate();
	bottomLeftPanel.validate();

	Dimension dim = upperLeftPanel.getSize();
	Insets ins = upperLeftPanel.getInsets();
	int width1 = dim.width - ins.left - ins.right;

	dim = bottomLeftPanel.getSize();
	ins = bottomLeftPanel.getInsets();
	int width2 = dim.width - ins.left - ins.right;

	Dimension newDim = new Dimension(width1 > width2 ? width1 : width2, 0);
	((Filler)rightStrut).changeShape(newDim,newDim,newDim);

	JLabel[] labs = {label1,label2,label3,label4,label5,label6,label7,label8,label9,label10,label11,label12};
	if (borderLabels != null && borderLabels.length == 12) {
	    for (int i=0; i<12; i++) {
		labs[i].setText(borderLabels[i]);
	    }
	} else {
	    for (int i=0; i<12; i++) {
		labs[i].setText(null);
	    }
	}

	leftSideLabels.validate();
	rightSideLabels.validate();

	dim = leftSideLabels.getSize();
	ins = leftSideLabels.getInsets();
	newDim = new Dimension(dim.width - ins.left - ins.right, 0);
	((Filler)leftStrut1).changeShape(newDim,newDim,newDim);
	((Filler)leftStrut2).changeShape(newDim,newDim,newDim);

	dim = rightSideLabels.getSize();
	ins = rightSideLabels.getInsets();
	newDim = new Dimension(dim.width - ins.left - ins.right, 0);
	((Filler)rightStrut1).changeShape(newDim,newDim,newDim);
	((Filler)rightStrut2).changeShape(newDim,newDim,newDim);
    }

    protected int getDivider() {

	if (numPoints > 40000)
	    return 10000;
	if (numPoints > 20000)
	    return 5000;
	if (numPoints > 10000)
	    return 2000;
	if (numPoints > 5000)
	    return 1000;
	if (numPoints > 2000)
	    return 500;
	if (numPoints > 1000)
	    return 200;
	if (numPoints > 500)
	    return 100;
	if (numPoints > 200)
	    return 50;
	if (numPoints > 100)
	    return 20;
	if (numPoints > 50)
	    return 10;
	if (numPoints > 20)
	    return 5;
	if (numPoints > 10)
	    return 2;

	return 1;
    }

    protected void calculateVerticalRange() {
	float max = 0;
	float min = 0;
	float diap;
	float offset = 0.1f;

        for(int q=0;q < multData.size();q++){
	    for (int i=0; i<curData.length; i++) {
		float[] tempFloat=(float[])multData.get(q);
		diap = tempFloat[i];
		if (diap < min)
		    min = diap;
		if (diap > max)
		    max = diap;
	    }
        }

	if (displayZeroFlag) {
	    if (min < 0) {
		if (max < 0) {
		    max = 0;
		}
	    } else {
		min = 0;
	    }
	}

	diap = (max - min) * offset;
	max += diap;
	min -= diap;

	vRangeMax = max;
	vRangeMin = min;
    }

    public void draw() {
	if (canvasImage != null ) {
	    if (dataType == TYPE_VECTOR) {	
		try{
		    VectorDraw.drawVectorData(display,canvasImage,multData,displayZeroFlag,vRangeMin,vRangeMax,colorSuite,displayErrorBars,noise);
		    drawVectorScale(colorSuite);
		}catch(IllegalAccessError e){
		    scalesPanel.remove(display);
		    VectorDraw.drawVectorData(display,canvasImage2,multData,displayZeroFlag,vRangeMin,vRangeMax,colorSuite,displayErrorBars,noise);
		    scalesPanel.add(display, "Center", -1);
		    canvasImage=canvasImage2;
		}
	    } else if (dataType == TYPE_BITMAP) {
		canvasImage2=canvasImage;
		canvasImage = BitmapDraw.drawBitmapData(display,canvasImage,bData);
	    }
	    display.repaint();
	    display.validate();
	}
    }

/* 
* Adds the x-axis if displayScale==true and displays the names of the
* data plots int the top right corner if displayDataLabels==true
*/
    protected void drawVectorScale(Color[] suite)	{
	if (displayScale) {
	    Graphics GR = canvasImage.getGraphics();
	    GR.setColor(suite[4]);
	    Font ft = GR.getFont();
	    FontMetrics fmtr = getFontMetrics(ft);
	    float coef = (float)(canvasWidth-21)/(float)(numPoints-1),
		  coef2 = (float)(canvasHeight-1)/(vRangeMax-vRangeMin);

	    int majorStripes = canvasHeight / 60;
	    if (majorStripes < 2)
		majorStripes = 2;

	    int linePosition = canvasHeight - (majorStripes / 2) - (fmtr.getHeight());
	    int majorStripesPos = linePosition - ((majorStripes * 2) / 3);
	    int minorStripesPos = linePosition - (majorStripes / 3);

	    GR.drawLine(0,linePosition,canvasWidth,linePosition);

	    if (coef > 3) {
		int j;
		for (int i=0; i<numPoints; i++) {
		    j = Math.round(coef * i+10f);
		    GR.drawLine(j,minorStripesPos,j,(minorStripesPos+(majorStripes/3)));
		}
	    }

	    if (xScaleLabels == null) {
		createScaleLabels();
	    }

	    String scaleLabel;
	    int labelWidth,margin,j;
	    for (int i=0; i<numPoints; i++) {
		j = startPoint + i;
		if ((j % getDivider()) == 0) {
		    scaleLabel = xScaleLabels[j];
		    labelWidth = fmtr.stringWidth(scaleLabel);
		    if (i == 0)
			margin = 1;
		    else if (i == (numPoints - 1))
			margin = (-1) - labelWidth;
		    else
			margin = 0 - (labelWidth/2);

		    j = Math.round(coef * i+10f);

		    int labelPos = j + margin;
		    if (labelPos > (canvasWidth - labelWidth - 1))
			labelPos = canvasWidth - labelWidth - 1;
		    if (labelPos < 1)
			labelPos = 1;

		    GR.drawLine(j,majorStripesPos,j,(majorStripesPos + majorStripes));
		    GR.drawString(scaleLabel,labelPos,(canvasHeight - 2));
		}
	    }
		double yGaps=Math.pow(10,(int)Math.floor(Math.log(vRangeMax-vRangeMin)/Math.log(10)));
		double gapMult=.1;
		if((vRangeMax-vRangeMin)/yGaps>2d){
			gapMult=(vRangeMax-vRangeMin)/yGaps>4d?.05:.02;
			yGaps*=10;
		}
		double top=vRangeMax-5/coef2;
		for(double i=displayZeroFlag?yGaps*gapMult:0d;i<top;i+=yGaps*gapMult){
			int convert=(int)((vRangeMax-i)*coef2);
			GR.drawLine(10,convert,15,convert);
			GR.drawString(String.valueOf((float)i),17,convert+5);
		}
		double linPos=vRangeMax-(linePosition-5)/coef2;
		for(double i=-yGaps*gapMult;i>linPos;i-=yGaps*gapMult){
			int convert=(int)((vRangeMax-i)*coef2);
			GR.drawLine(10,convert,15,convert);
			GR.drawString(String.valueOf((float)i),17,convert+5);
		}
		GR.drawLine(10,0,10,linePosition);
	}
	if(displayDataLabels){
	    Graphics GR=canvasImage.getGraphics();
	    Font ft=GR.getFont();
	    FontMetrics fmtr=getFontMetrics(ft);

	    int colorNumber=1;
	    for(int i=0;i<dataLabels.length;i++){
		GR.setColor(suite[colorNumber]);
		GR.drawString(dataLabels[i],(canvasWidth-150),(10*(i+1)));
		colorNumber++;
		if(colorNumber==4){
		    colorNumber++;
		}
		if(colorNumber==6){
		    colorNumber=1;
		}
	    }
	}
    }

    public void setColorSuite(Color[] newSuite) {
	colorSuite = newSuite;
	draw();
    }

    // horizontal line stuff
    public void setHorizontalLine( int newLineNumber, float newLineValue )  {
	if((newLineNumber >= 0) && (newLineNumber < 10)) {
	    hTLines[newLineNumber] = newLineValue;
	    hTLFlag[newLineNumber] = true;
	    hTLColor[newLineNumber] = Color.yellow;
	}
	display.repaint();
    }

    public void setHorizontalLine( int newLineNumber, float newLineValue, Color newLineColor )  {
	if((newLineNumber >= 0) && (newLineNumber < 10)) {
	    hTLines[newLineNumber] = newLineValue;
	    hTLFlag[newLineNumber] = true;
	    hTLColor[newLineNumber] = newLineColor;
	}
	display.repaint();
    }

    public float getHorizontalLine( int whichLine )  {
	if((whichLine >= 0) && (whichLine < 10))
	    return hTLines[whichLine];
	else
	    return 0f;
    }

    public void removeHorizontalLine( int whichLine )  {
	if((whichLine >= 0) && (whichLine < 10))
	    hTLFlag[whichLine] = false;
	display.repaint();
    }

    public void removeAllHorizontalLines()  {
	for(int i=0; i<10; i++)
	    hTLFlag[i] = false;
	display.repaint();
    }

    public void setDataScale(float min,float max) {
	if (dataType == TYPE_VECTOR) {
	    if (min <= max) {
		vRangeMin = min;
		vRangeMax = max;
	    }
	    // this draw call is here as a precaution - it may not be neccessary
	    draw();
	}
    }

    public float[] getDataScale() {
	calculateVerticalRange();
	float[] res = new float[2];
	res[0] = vRangeMin;
	res[1] = vRangeMax;
	return res;
    }

    protected void handleResizeEvent() {
	Dimension dim = display.getSize();
	Insets ins = display.getInsets();
	if(dim.width <= 0)
	    dim.width = 10;
	if(dim.height <= 0)
	    dim.height = 10;
	canvasWidth = dim.width - ins.left - ins.right;
	canvasHeight = dim.height - ins.top - ins.bottom;
	canvasImage = null;
    }

    protected void createScaleLabels() {
	xScaleLabels = null;
	yScaleLabels = null;

	if (scaleX == null) {
	    scaleX = new String[vData.length];
	    for (int i=0; i<scaleX.length; i++) {
		scaleX[i] = String.valueOf(i);
	    }
	}

	xScaleLabels = new String[scaleX.length];
	int k;

	try{
	    for (int i=0; i<vData.length; i++) {
		k = Math.round(Integer.parseInt(scaleX[i]) * 100);
		xScaleLabels[i] = String.valueOf((float)k/100);
	    }
	}catch(Exception e){
	    xScaleLabels=scaleX;
	}
    }

    // cursor stuff
    public void setCursorPosition(int newX,int newY) {
  	if (newX == -1 && newY == -1) {
	}else {
	    if (dataType == TYPE_VECTOR) {
		if (newX < 0)
		    vectorPoint = 0;
		else if (newX > vData.length)
		    vectorPoint = vData.length - 1;
		else
		    vectorPoint = newX;

		float coef = (float)(canvasWidth-1) / (float)(numPoints-1);
		mousePoint.x = Math.round((float)(vectorPoint - startPoint) * coef);
		mousePoint.y = newY;
	    }
	    else if (dataType == TYPE_BITMAP) {
		bitmapCursorPosition.x = newX;
		bitmapCursorPosition.y = newY;

		float coef = (float)(canvasWidth-1)/(float)(bitmapWidth-1);
		float coef2 = (float)(canvasHeight-1)/(float)(bitmapHeight-1);

		mousePoint.x = Math.round((float)bitmapCursorPosition.x*coef) + (canvasWidth/(bitmapWidth*2));
		mousePoint.y = Math.round((float)bitmapCursorPosition.y*coef2) + (canvasHeight/(bitmapHeight*2));
	    }
	    else if (dataType == TYPE_NODATA) {
		mousePoint.x = newX;
		mousePoint.y = newY;
	    }
	}
	display.repaint();
    }


    // boolean methods, self-explanatory
    public void setDisplayZeroFlag(boolean newFlag) {
	displayZeroFlag = newFlag;
	//draw();
    }

    public boolean getDisplayZeroFlag() {
	return displayZeroFlag;
    }

    public void setDrawCoordinateLines(boolean newState) {
	drawCoordinateLines = newState;
	display.repaint();
    }

    public boolean getDrawCoordinateLines() {
	return drawCoordinateLines;
    }

    public void setDisplayCoordinates(boolean newState) {
	displayCoordinates = newState;
	display.repaint();
    }

    public boolean getDisplayCoordinates() {
	return displayCoordinates;
    }
    
    public void setDisplayDataLabels(boolean newState){
	displayDataLabels=newState;
	display.repaint();
    }

    public boolean getDisplayDataLabels(){
	return displayDataLabels;
    }

    public void setDisplayDataNames(boolean newState){
	displayDataNames=newState;
    }

    public boolean getDisplayDataNames(){
	return displayDataNames;
    }

    public void setDisplayErrorBars(boolean newState, int i) {
	displayErrorBars[i]=newState;
	display.repaint();
    }

    // vector axis setting - returning
    public void setVectorXAxis( int newStart, int newEnd )  {
	startPoint = newStart;
	endPoint = newEnd;
	numPoints = newEnd - newStart + 1;
    }

    public int[] getVectorXAxis()  {
       	int[] res = new int[3];
       	res[0] = startPoint;
       	res[1] = endPoint;
       	res[2] = numPoints;
       	return res;
    }

    //  listeners
    public synchronized void addPointChangedListener( PointChangedListener newListener )  {
        if(pointListeners == null)
            pointListeners = new Vector<PointChangedListener>();
        pointListeners.addElement( newListener );
    }

    public synchronized void removePointChangedListener( PointChangedListener theListener )  {
        if(pointListeners != null) {
            if(pointListeners.contains( theListener ))
                pointListeners.removeElement( theListener );
            else
                throw( new NoSuchElementException() );
            if(pointListeners.size() == 0)
                pointListeners = null;
        } else
            throw( new NoSuchElementException() );
    }

    protected void firePointChangedEvent()  {
        if(pointListeners != null) {
            PointChangedEvent e = null;
            if(dataType == TYPE_VECTOR)
                e = new PointChangedEvent( this,vectorPoint, mousePoint.y );
            else if(dataType == TYPE_BITMAP)
                e = new PointChangedEvent( this,bitmapCursorPosition.x, bitmapCursorPosition.y );
            else if(dataType == TYPE_NODATA)
                e = new PointChangedEvent( this, mousePoint.x, mousePoint.y );
            Vector v;
            synchronized(this) {
                v = (Vector)pointListeners.clone();
            }
            for(int i=0; i<v.size(); i++) {
                PointChangedListener pcl =
                    (PointChangedListener)(v.elementAt(i));
                pcl.pointChanged( e );
            }
        }
    }

    public synchronized void addMarkListener( MarkListener newListener )  {
        if(markListeners == null)
            markListeners = new Vector<MarkListener>();
        markListeners.addElement( newListener );
    }

    public synchronized void removeMarkListener( MarkListener theListener )  {
        if(markListeners != null) {
            if(markListeners.contains( theListener ))
                markListeners.removeElement( theListener );
            else
                throw( new NoSuchElementException() );
            if(markListeners.size() == 0)
               markListeners = null;
        } else
            throw( new NoSuchElementException() );
    }

    protected void fireMarkEvent(boolean rel)  {
	if(markListeners != null) {
            MarkEvent e;
            if(dataType == TYPE_VECTOR) {
		e = new MarkEvent(this,new Point(mousePoint),MarkEvent.VECTOR,rel);
            } else if(dataType == TYPE_BITMAP)
                e = new MarkEvent( this, new Point(mousePoint),MarkEvent.BITMAP,rel);
            else
                e = new MarkEvent( this, new Point(mousePoint), -1,rel);
            Vector v;
            synchronized(this) {
                v = (Vector)markListeners.clone();
            }
            for(int i=0; i<v.size(); i++) {
                MarkListener ml = (MarkListener)(v.elementAt( i ));
                ml.mark( e );
	    }
	}
    }

    public synchronized void addZoomListener( ZoomListener newListener )  {
        if(zoomListeners == null)
            zoomListeners = new Vector<ZoomListener>();
        zoomListeners.addElement( newListener );
    }

    public synchronized void removeZoomListener( ZoomListener theListener )  {
        if(zoomListeners != null) {
            if(zoomListeners.contains( theListener ))
                zoomListeners.removeElement( theListener );
            else
                throw( new NoSuchElementException() );
            if(zoomListeners.size() == 0)
                zoomListeners = null;
        } else
            throw( new NoSuchElementException() );
    }

    protected void fireZoomEvent(boolean b)  {
	if(zoomListeners != null) {
            ZoomEvent e;
            if(dataType == TYPE_VECTOR) {
		e = new ZoomEvent(this,mousePoint,ZoomEvent.VECTOR,b);
            } else if(dataType == TYPE_BITMAP)
                e = new ZoomEvent( this, new Point(mousePoint),ZoomEvent.BITMAP,b);
            else
                e = new ZoomEvent( this, new Point(mousePoint), -1,b);
            Vector v;
            synchronized(this) {
                v = (Vector)zoomListeners.clone();
            }
            for(int i=0; i<v.size(); i++) {
                ZoomListener zl = (ZoomListener)(v.elementAt( i ));
                zl.zoom( e );
            }
	}
    }

    // highlighting and region marking
    public void startHighlight(Point a,int x) {
	Graphics g = canvasImage.getGraphics();
	g.setColor(new Color(192,192,192,75));

	if (x == 0) {
	    g.drawLine(a.x,0,a.x,canvasHeight);
	} else if (x == 1) {
	    g.drawLine(a.x,a.y,a.x,a.y);
	}
	display.repaint();
    }

    public void endHighlight(Point a,Point b) {
	if (a.x > b.x) {
	    a.x += b.x;
	    b.x = a.x - b.x;
	    a.x = a.x - b.x;
	}

	Graphics g = canvasImage.getGraphics();
	g.setColor(new Color(192,192,192,75));
	g.fillRect(a.x,0,b.x-a.x,canvasHeight);
	display.repaint();
    }

    public void setFill(boolean arg,Point a,Point b) {
	fillIn = arg;
	if (a == null) {
	    fillB = b;
	} else {
	    fillA = a;
	    fillB = a;
	}
	display.repaint();
    }


    // grid controls
    public void makeGrid(int a, int b) {
	grid = true;
	numbXLines = a;
	numbYLines = b;
	display.repaint();
    }

    public void destroyGrid() {
	grid = false;
	numbXLines = 0;
	numbYLines = 0;
	display.repaint();
    }

    public void selectGridLines(int a, int b) {
	xSel = a;
	ySel = b;
	display.repaint();
    }

    public void offsetGrids(float xoff, float yoff) {
	xOffset = xoff;
	yOffset = yoff;
	display.repaint();
    }

    // zooming
    public void zoom( int zType, Point stPt, Point endPt )  {
	if(dataType == TYPE_VECTOR) {
	    if (zType == ZOOM_RESTORE) {
		startPoint = 0;
		curData = vData;
		numPoints = curData.length;
		endPoint = numPoints - 1;
		createScaleLabels();
		calculateVerticalRange();
	    } else {
		int tmp[] = DrawingMethods.getRectangle(stPt, endPt, endPoint);
		startPoint = tmp[0];
		endPoint = tmp[2];
		numPoints = endPoint - startPoint + 1;
		float[] temp = new float[numPoints];
		for (int i=0; i<numPoints; i++) {
		    temp[i] = vData[i+startPoint];
		}
		curData = temp;
		if (zType == ZOOM_TOFIT) {
		} else if (zType == ZOOM_KEEPSCALE) {
		} else if (zType == ZOOM_RECTANGLE) {
		    float coef = (vRangeMax - vRangeMin) / (canvasHeight - 1);
		    vRangeMax = (vRangeMax - ((float)tmp[1] * coef));
		    vRangeMin = (vRangeMax - ((float)tmp[3] * coef));
		}
	    }
	}
	fillIn = false; // to erase the zoom marks
	draw();
    }

    public synchronized void addActiveICChangedListener( ActiveICChangedListener newListener )  {
	if(aICCListeners == null)
	    aICCListeners = new Vector<ActiveICChangedListener>();
	aICCListeners.addElement( newListener );
    }

    public synchronized void removeActiveICChangedListener( ActiveICChangedListener theListener )  {
	if(aICCListeners != null) {
	    if(aICCListeners.contains( theListener )) {
		aICCListeners.removeElement( theListener );
	    } else {
		throw( new NoSuchElementException() );
	    }

	    if(aICCListeners.size() == 0)
		aICCListeners = null;
	} else
	    throw( new NoSuchElementException() );
    }

    protected void fireActiveICChangedEvent(boolean b)  {
	if(aICCListeners != null) {
	    ActiveICChangedEvent e = new ActiveICChangedEvent(this,b);
	    Vector v;
	    synchronized(this) {
		v = (Vector)aICCListeners.clone();
	    }
	    for(int i=0; i<v.size(); i++) {
		ActiveICChangedListener al = (ActiveICChangedListener)(v.elementAt(i));
		al.activeIC(e);
	    }
	}
    }

    public void setFocused( boolean newFocusStatus )  {
        focusFlag = newFocusStatus;
        if(focusFlag)
            display.setBorder(javax.swing.BorderFactory.createLineBorder(Color.red,3));
        else {
	    display.setBorder(javax.swing.BorderFactory.createLineBorder(colorSuite[8],3));
	}
	repaint();
    }

    public boolean isFocused()  {
        return focusFlag;
    }

    public int print(Graphics g, PageFormat pf, int index) {
	if (index != 0) {
	    return NO_SUCH_PAGE;
	} else {
	    Color[] tSuite={Color.white,Color.blue,Color.red,Color.orange,Color.black,Color.gray,Color.black,Color.blue,Color.black};
	    VectorDraw.drawVectorData(display,canvasImage,multData,displayZeroFlag,vRangeMin,vRangeMax,tSuite,displayErrorBars,noise);
	    drawVectorScale(tSuite);
	    Graphics2D g2d = (Graphics2D)g;
	    g2d.translate(pf.getImageableX(), pf.getImageableY());
	    g2d.scale(pf.getImageableWidth()/canvasWidth,pf.getImageableHeight()/canvasHeight);
	    RepaintManager paintManager=RepaintManager.currentManager(this);
	    paintManager.setDoubleBufferingEnabled(false);
	    this.paint(g2d);
	    paintManager.setDoubleBufferingEnabled(true);
	    return PAGE_EXISTS;
	}
    }

    protected Point translate(Point e) {
	if (dataType == TYPE_VECTOR) {
	    int x = ((curData.length * e.x)/canvasWidth) + startPoint;
	    return new Point(x,e.y);
	} else {
	    return e;
	}
    }

    protected class canvasPanel extends JPanel  {
        protected boolean dragFlag = false;

        public canvasPanel() {
	    super( null, false );
	    setDoubleBuffered( false );
	    setOpaque(false);
	    addComponentListener( new ComponentListenerAdapter() );
	    addMouseListener( new MouseListenerAdapter() );
	    addMouseMotionListener( new MouseMotionAdapter() );
        }

        public void paintComponent(Graphics compGR)  {
	    Graphics GR = compGR.create();
	    Insets ins = getInsets();

	    if(canvasImage == null)  {
		Dimension size = getSize();

		canvasImage = createImage( size.width, size.height );
		canvasWidth = size.width - ins.left - ins.right;
		canvasHeight = size.height - ins.top - ins.bottom;
		draw();
	    }

	    GR.clearRect(0,0,canvasWidth,canvasHeight);
	    GR.drawImage( canvasImage, ins.left, ins.top, this );

	    if (fillIn) {
		DrawingMethods.fillIn(GR,fillA,fillB);
		fillIn = false;
	    }

	    if (grid) { // grid drawing
		if (dataType == TYPE_BITMAP) {
		    DrawingMethods.drawGrid(GR,canvasWidth,canvasHeight,numbXLines,numbYLines,xSel,ySel,xOffset,yOffset);
		}
	    }

	    if(drawCoordinateLines) {
		DrawingMethods.drawCoordLines(GR,ins,canvasWidth,canvasHeight,mousePoint,colorSuite[3]);
	    }

	    // horizontal line drawing
	    int j;
	    float dY = (float)(canvasHeight - 1) / (vRangeMax - vRangeMin);
	    for(int i=0; i<10; i++) {
		if(hTLFlag[i]) {
		    j = Math.round(dY * (vRangeMax - hTLines[i]));
		    GR.setColor( hTLColor[i] );
		    GR.drawLine( 0, j, canvasWidth, j );
		}
	    }

	    if(displayCoordinates||displayDataNames) {
		if (dataType == TYPE_VECTOR && xScaleLabels != null) {
		    FontMetrics fmt = getFontMetrics(new Font("Monospaced", Font.PLAIN, 14));
		    int tempFloat=100000;  //distance to closest point
		    int tempInt=0;  //closest point
		    int holder;
		    for(int i=0;i<multData.size();i++){  //finds the closest data point to the mouse and the distance to it
			holder=Math.abs(Math.round(((float)((((Dimension)display.getSize()).height
				-((Insets)display.getInsets()).top-((Insets)display.getInsets()).bottom)-1)
				/(vRangeMax-vRangeMin))*(vRangeMax-((float[])multData.get(i))[vectorPoint]))
				-mousePoint.y);
			if(holder<=tempFloat){
			    tempFloat=holder;
			    tempInt=i;
			}
		    }
		    if(tempFloat<10){  //displays the data name if the point is within 10 pixels
			if(displayCoordinates){
			    DrawingMethods.displayCoordinates(GR,ins,fmt,canvasWidth,xScaleLabels,((float[])multData.get(tempInt)),vectorPoint,mousePoint,colorSuite);
			}
			if(displayDataNames){
			    DrawingMethods.displayDataNames(GR,ins,fmt,canvasWidth,xScaleLabels,dataNames[tempInt],vectorPoint,mousePoint,colorSuite);
			}
		    }
		}
	    }
	    GR.dispose();
        }

        protected void calculatePosition(Point newPos) {
            float coef, coef2;
            Insets ins = getInsets();

            if (dataType == TYPE_VECTOR) {
                coef = (float)(canvasWidth - 1) / (float)(numPoints - 1);
                vectorPoint = Math.round((float)(newPos.x - ins.left) / coef);
                if(vectorPoint < 0) {
                    vectorPoint = 0;
                } else if(vectorPoint > endPoint) {
                    vectorPoint = endPoint;
                }
                mousePoint.x = Math.round((float)vectorPoint * coef);
                mousePoint.y = newPos.y - ins.top;
		actualMousePoint = new Point(newPos);
                vectorPoint += startPoint;
            } else if (dataType == TYPE_BITMAP) {
                coef = (float)(canvasWidth - 1) / (float)(bitmapWidth - 1);
                coef2 = (float)(canvasHeight - 1) / (float)(bitmapHeight - 1);
                bitmapCursorPosition.x = Math.round((float)newPos.x / coef);
                bitmapCursorPosition.y = Math.round((float)newPos.y / coef2);
                mousePoint.x = Math.round((float)bitmapCursorPosition.x * coef) + (canvasWidth / (bitmapWidth * 2));
                mousePoint.y = Math.round((float)bitmapCursorPosition.y * coef2) + (canvasHeight / (bitmapHeight * 2));
            } else if(dataType == TYPE_NODATA) {
                mousePoint.x = newPos.x;
                mousePoint.y = newPos.y;
            }
        }

        class ComponentListenerAdapter implements ComponentListener  {
            public void componentResized( ComponentEvent e )  {
                if ( e.getSource().equals( canvasPanel.this ) )  {
                    ImageCanvas.this.handleResizeEvent();
                    return;
                }
            }
            public void componentShown( ComponentEvent e )  {
            }
            public void componentHidden( ComponentEvent e )  {
            }
            public void componentMoved( ComponentEvent e )  {
            }
        }

        class MouseListenerAdapter implements MouseListener {

            public void mouseClicked( MouseEvent e ) {
		//fireActiveICChangedEvent();
		dragFlag = false;
            }

            public void mouseEntered( MouseEvent e ) {
                dragFlag = false;
            }

            public void mouseExited( MouseEvent e ) {
                dragFlag = false;
            }

            public void mousePressed( MouseEvent e ) {
		mousePoint = e.getPoint();
		fireMarkEvent(true);
		mousePoint = translate(e.getPoint());
		fireZoomEvent(true);
	    }

            public void mouseReleased( MouseEvent e ) {
		if(dragFlag) {
		    mousePoint = e.getPoint();
		    fireMarkEvent(true);
		    mousePoint = translate(e.getPoint());
		    fireZoomEvent(true);
		    dragFlag = false;
		} else {
		    boolean tmp = false;
		    if (e.isShiftDown())
			tmp = true;

		    fireActiveICChangedEvent(tmp);
		    dragFlag = false;
		}
	    }
	}

	class MouseMotionAdapter implements MouseMotionListener {
	    public void mouseMoved(MouseEvent e) {
		calculatePosition(e.getPoint());
		firePointChangedEvent();
		repaint();
	    }

	    public void mouseDragged(MouseEvent e) {
	       	mousePoint = e.getPoint();
		mouseStartPoint = mousePoint;
		dragFlag = true;
		fireMarkEvent(false);
	    }
	}
    }
}
