// Class: VectorDraw
//
// Purpose:  written to seperate image construction from image display for vector objects
//
//  History: 11/99 jac
//
// COPYRIGHT, 2000, Fox Chase Cancer Center
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


package viewer.csi.gui;

import java.awt.Image;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Color;
import java.lang.String;
import java.lang.Float;
import java.lang.Math;
import javax.swing.JPanel;
import java.util.*;

public class VectorDraw implements viewer.csi.gui.csiConstants {

    public static int version = 200001;
    public static String copyright = "Copyright 2000, Fox Chase Cancer Center";

    // vector variables
    private static int numPoints;
    private static float[] data;
    private static int width, height;
    private static float vRangeMax, vRangeMin;
    private static Image im;
    private static JPanel canPanel;
    private static boolean dispZero = false;

    public static void drawVectorData(JPanel newPanel,Image newImage, ArrayList multiData, boolean b, float vMin, float vMax, Color[] suite, boolean[] displayErrorBars, float[] noise) {
	canPanel = newPanel;
	im = newImage;
        int colorNumber=1;
	data=(float[])multiData.get(0);

	vRangeMin = vMin;
	vRangeMax = vMax;
	Dimension dim = canPanel.getSize();
	Insets ins = canPanel.getInsets();
	if(dim.width <= 0)
	    dim.width = 10;
	if(dim.height <= 0)
	    dim.height = 10;
	width = dim.width - ins.left - ins.right;
	height = dim.height - ins.top - ins.bottom;

	numPoints = data.length;
	dispZero = b;

	// draw the data
	int[] xArray,yArray,noiseArray;

	int points = numPoints;

        Graphics GR = im.getGraphics();
	GR.clearRect(0,0,width,height);
	GR.setColor( suite[0] );
	GR.fillRect(0,0,width,height);

	float coef = (float)(width-21)/(float)(numPoints-1),
	    coef2 = (float)(height-1)/(vRangeMax-vRangeMin);

        for( int q=0; q < multiData.size();q++){
	    data = (float[])multiData.get(q);

	    // workaround for drawPolyline, which crashes if > 8k points
	    if (numPoints > 8192) {
		int skip = (int)Math.ceil((double)numPoints/(double)8192);
		points = numPoints/skip;
		xArray = new int[points];
		yArray = new int[points];
		noiseArray = new int[points];
		int j = 0;
		for (int i=0; i<numPoints; i+=skip) {
		    xArray[j] = Math.round(coef * j * skip+10f);
		    yArray[j] = Math.round(coef2 * (vRangeMax - data[i]));
		    if(displayErrorBars[q]){
			noiseArray[j] = Math.round(coef2 * (vRangeMax - noise[i]));
		    }
		    j++;
		}
		GR.setColor(suite[colorNumber]);
		GR.drawPolyline(xArray,yArray,points);
		if(displayErrorBars[q]){
		    for(int x=0;x<numPoints;x++){
			GR.drawLine(xArray[x],yArray[x]-noiseArray[x],xArray[x],yArray[x]+noiseArray[x]);
			GR.drawLine(xArray[x]-5,yArray[x]-noiseArray[x],xArray[x]+5,yArray[x]-noiseArray[x]);
			GR.drawLine(xArray[x]-5,yArray[x]+noiseArray[x],xArray[x]+5,yArray[x]+noiseArray[x]);
		    }
		}
	    } else { // if less than 8k points, basic code
		xArray = new int[numPoints];
		yArray = new int[numPoints];
		noiseArray = new int[points];
		int j = 0;
		for (int i=0; i<numPoints; i++) {
		    xArray[j] = Math.round(coef * j+10f);
		    yArray[j] = Math.round(coef2 * (vRangeMax - data[i]));
		    if(displayErrorBars[q]){
			noiseArray[j] = Math.round(coef2 * (noise[i]));
		    }
		    j++;
		}
		GR.setColor(suite[colorNumber]);
		GR.drawPolyline(xArray,yArray,numPoints);
		if(displayErrorBars[q]){
		    for(int x=0;x<numPoints;x++){
			GR.drawLine(xArray[x],yArray[x]-noiseArray[x],xArray[x],yArray[x]+noiseArray[x]);
			GR.drawLine(xArray[x]-5,yArray[x]-noiseArray[x],xArray[x]+5,yArray[x]-noiseArray[x]);
			GR.drawLine(xArray[x]-5,yArray[x]+noiseArray[x],xArray[x]+5,yArray[x]+noiseArray[x]);
		    }
		}
	    }
	    if(colorNumber>4){
		colorNumber=0;
	    }
	    colorNumber++;
	    if(colorNumber==4){
		colorNumber++;
	    }
        }

	// if flag is set, draw zero line
	if (dispZero) {
	    GR.setColor(suite[4]);
	    int j = Math.round(coef2 * vRangeMax);
	    GR.drawLine(0,j,width,j);
	}
    }
}


































